﻿''' <summary>
''' マウスカーソルからの逃避行 ～ 派生クラス：『画面の中の金魚』編
''' </summary>
''' <remarks>マウスカーソルから一定距離を置いて帯同する（金魚の糞）。</remarks>
Public Class Avoid_Randezvous

    Inherits Avoid_Base



    '===========================================================================================================



    ''' <summary>画面右端からのマージン</summary>
    Private HorizonMargin As Integer

    ''' <summary>画面下端からのマージン</summary>
    Private VerticalMargin As Integer



    '===========================================================================================================



    ''' <summary>
    ''' インスタンスの初期化
    ''' </summary>
    ''' <param name="Viewer">ビューア</param>
    ''' <remarks></remarks>
    Public Sub New(Viewer As FormMain_Viewer)

        MyBase.New(Viewer)

    End Sub

    Public Overrides Sub Initialize(WorkArea As Rectangle)

        '=== 初期値設定 ===
        AvoidSpeed = INITIAL_SPEED
        AvoidExerciseDistance = Math.Min(WorkArea.Width, WorkArea.Height) / 100 * SettingData.Lupe.Avoid.ExerciseDistance '回避発動距離は、画面の縦横の短い方を基準として算出。

        '--- マージン ---
        HorizonMargin = Owner.Width + AvoidExerciseDistance
        VerticalMargin = Owner.Height + AvoidExerciseDistance

        '--- 最初の位置を確定 ---
        Plan(WorkArea)

    End Sub

    Protected Overrides Sub Plan(WorkArea As Rectangle)

        Static relativePosition As Size '前回のマウスカーソル位置に対する相対的な位置
        Dim mousePosition As Point = Windows.Forms.Cursor.Position 'マウスカーソルの位置

        '=== 移動先 ===
        Dim tempRelative As New Size(If(mousePosition.X > WorkArea.Right - HorizonMargin, -HorizonMargin, AvoidExerciseDistance),
                                     If(mousePosition.Y > WorkArea.Bottom - VerticalMargin, -VerticalMargin, AvoidExerciseDistance)) '現在のマウスカーソルに対する相対的な位置
        'マウスカーソルが画面右限界線を越えたら、マウスカーソルの左側へ、そうでなければ右側へ
        'マウスカーソルが画面下限界線を越えたら、マウスカーソルの上側へ、そうでなければ下側へ

        '--- 位置取り確認 ---
        If Not relativePosition.Equals(tempRelative) Then
            relativePosition = tempRelative
            AvoidSpeed = INITIAL_SPEED 'マウスカーソルに対する位置取りが以前と違う（画面端に近づいた）場合は、回避速度が速いままだと一気に飛んでしまうので初速に戻す。
        End If

        '--- 移動先確定 ---
        AvoidDestination = mousePosition + tempRelative

        '画面越境しないなら移動先を使用画面内に収める
        If Not SettingData.Lupe.Avoid.Cross Then
            With AvoidDestination
                If .X < WorkArea.Left Then
                    .X = WorkArea.Left
                ElseIf .X > WorkArea.Right - Owner.Width Then
                    .X = WorkArea.Right - Owner.Width
                End If
                If .Y < WorkArea.Top Then
                    .Y = WorkArea.Top
                ElseIf .Y > WorkArea.Bottom - Owner.Height Then
                    .Y = WorkArea.Bottom - Owner.Height
                End If
            End With
        End If

        '=== 到着 ===
        If AvoidDestination = Owner.Location AndAlso AvoidSpeed <> INITIAL_SPEED Then AvoidSpeed = INITIAL_SPEED

    End Sub

End Class
