﻿''' <summary>
''' マウスカーソルからの逃避行 ～ 派生クラス：『画面右の三角関係』編
''' </summary>
''' <remarks>画面の右の3地点を行ったり来たりする。</remarks>
Public Class Avoid_RightTriangle

    Inherits Avoid_Base



    '===========================================================================================================



    ''' <summary>安寧の地</summary>
    Private Eden As Point

    ''' <summary>回避喚起地帯</summary>
    Private Zone As Rectangle



    '===========================================================================================================



    ''' <summary>
    ''' インスタンスの初期化
    ''' </summary>
    ''' <param name="Viewer">ビューア</param>
    ''' <remarks></remarks>
    Public Sub New(Viewer As FormMain_Viewer)

        MyBase.New(Viewer)

    End Sub

    Public Overrides Sub Initialize(WorkArea As Rectangle)

        '=== 初期値設定 ===
        With WorkArea

            AvoidSpeed = INITIAL_SPEED
            AvoidExerciseDistance = Math.Min(.Width, .Height) / 100 * SettingData.Lupe.Avoid.ExerciseDistance '回避発動距離は、画面の縦横の短い方を基準として算出。

            '--- パラメータ ---
            Eden = New Point(.Right - Owner.Width, .Top + (.Height - Owner.Height) \ 2) '画面右中央（基本位置）
            Zone = New Rectangle(.Right - Owner.Width - AvoidExerciseDistance, .Top, Owner.Width + AvoidExerciseDistance * 2, .Height)

            '--- とりあえずの表示位置 ---
            AvoidDestination = Eden '右中央

        End With

    End Sub

    Protected Overrides Sub Plan(WorkArea As Rectangle)

        Dim newLocate As Point '新しい目的地
        Dim mousePosition As Point = Windows.Forms.Cursor.Position 'マウスカーソルの位置

        '=== 目的地 ===
        With WorkArea

            If Zone.Contains(mousePosition) Then

                '--- 画面の上下中央へ出張 ---
                newLocate.X = .Left + (.Width - Owner.Width) \ 2 '画面中央

                If AvoidDestination = Eden Then

                    '初出張の場合・・・マウスカーソルが、下半分にあれば上へ、上半分にあれば下へ
                    newLocate.Y = If(mousePosition.Y > .Top + .Height \ 2, .Top, .Bottom - Owner.Height)

                Else

                    '出張中の場合・・・マウスカーソルが、ビューアの許容範囲内に入ったか？
                    If mousePosition.Y > .Bottom - Owner.Height - AvoidExerciseDistance Then
                        newLocate.Y = .Top '上へ
                    ElseIf mousePosition.Y < .Top + Owner.Height + AvoidExerciseDistance Then
                        newLocate.Y = .Bottom - Owner.Height '下へ
                    Else
                        newLocate.Y = AvoidDestination.Y '現状維持
                    End If

                End If

            Else

                '--- 安寧の地へ ---
                newLocate = Eden

            End If

        End With

        '=== 目的地変更 ===
        If AvoidDestination <> newLocate Then
            AvoidDestination = newLocate
            AvoidSpeed = INITIAL_SPEED
        End If

    End Sub

End Class
