﻿''' <summary>
''' マウスカーソルからの逃避行 ～ 派生クラス：『画面四隅のすれ違い』編
''' </summary>
''' <remarks>画面の四隅をグルグルする。</remarks>
Public Class Avoid_ScreenCorner

    Inherits Avoid_Base



    '===========================================================================================================



    ''' <summary>画面左右端からの許容範囲</summary>
    Private HorizonMargin As Integer

    ''' <summary>画面上下端からの許容範囲</summary>
    Private VerticalMargin As Integer



    '===========================================================================================================



    ''' <summary>
    ''' インスタンスの初期化
    ''' </summary>
    ''' <param name="Viewer">ビューア</param>
    ''' <remarks></remarks>
    Public Sub New(Viewer As FormMain_Viewer)

        MyBase.New(Viewer)

    End Sub

    Public Overrides Sub Initialize(WorkArea As Rectangle)

        '=== 初期値設定 ===
        With WorkArea

            AvoidSpeed = INITIAL_SPEED
            AvoidExerciseDistance = Math.Min(.Width, .Height) / 100 * SettingData.Lupe.Avoid.ExerciseDistance '回避発動距離は、画面の縦横の短い方を基準として算出。

            '--- 許容範囲 ---
            HorizonMargin = Owner.Width + AvoidExerciseDistance
            VerticalMargin = Owner.Height + AvoidExerciseDistance

            '--- とりあえずの表示位置 ---
            AvoidDestination.X = If(.Left + (.Width - Owner.Width) / 2 > Owner.Left, .Left, .Right - Owner.Width) '左右のどちらか近い方。
            AvoidDestination.Y = If(.Top + (.Height - Owner.Height) / 2 > Owner.Top, .Top, .Bottom - Owner.Height) '上下のどちらか近い方。

        End With

    End Sub

    Protected Overrides Sub Plan(WorkArea As Rectangle)

        Dim newLocate As Point '新しい目的地
        Dim mousePosition As Point = Windows.Forms.Cursor.Position 'マウスカーソルの位置

        '=== 目的地 ===
        With WorkArea
            If mousePosition.X < .Left + HorizonMargin Then
                newLocate.X = .Right - Owner.Width '画面右端へ
            ElseIf mousePosition.X > .Right - HorizonMargin Then
                newLocate.X = .Left '画面左端へ
            Else
                newLocate.X = AvoidDestination.X '現状維持
            End If

            If mousePosition.Y < .Top + VerticalMargin Then
                newLocate.Y = .Bottom - Owner.Height '画面下端へ
            ElseIf mousePosition.Y > .Bottom - VerticalMargin Then
                newLocate.Y = .Top '画面上端へ
            Else
                newLocate.Y = AvoidDestination.Y '現状維持
            End If
        End With

        '=== 目的地変更 ===
        If AvoidDestination <> newLocate Then
            AvoidDestination = newLocate
            AvoidSpeed = INITIAL_SPEED
        End If

    End Sub

End Class
