﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『ボタンにカラーボックスを表示する方法』
'  │・予めBitmapを作成しておき、そこに任意の色で四角形を描き、それを[Button]コントロールの[Image]プロパティに入れて再描画しているだけ。
'  │・ただし、書き換えるたびに[Image]プロパティに登録しなおさないと、なぜか反映されないみたい。
'  │
'  └────────────





Imports System.ComponentModel

''' <summary>
''' カラーボックス付きボタン
''' </summary>
''' <remarks>Buttonコントロールを継承する</remarks>
Public Class ColorBoxButton

    Inherits Button


    Private Bmp As Bitmap, Grp As Graphics 'カーラーボックスの実体

    Private _BoxSize As Size = New Size(11, 11)  'カラーボックスのサイズ（11 = 《Me.FontHeight - 1》(9pt時)を採用）
    Private _SlashColor As Color = Color.White 'カラーボックスに引く斜線の色
    Private _BoxColor As Color = Color.Black 'カラーボックスの色


    Public Sub New()

        MyBase.New()
        MakeColorBox()

    End Sub

    ''' <summary>
    ''' カーラーボックスのサイズ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("カラーボックス")> _
    <Description("カラーボックスのサイズを指定します。")> _
    <DefaultValue(GetType(Size), "11, 11")> _
    Public Property ColorBoxSize() As Size

        Get
            Return _BoxSize
        End Get

        Set(ByVal value As Size)
            _BoxSize = value
            MakeColorBox()
        End Set

    End Property

    ''' <summary>
    ''' ボックスの斜線色
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("カラーボックス")> _
    <Description("ボックスに引く斜線の色を指定します。")> _
    <DefaultValue(GetType(Color), "White")> _
    Public Property SlashColor() As Color

        Get
            Return _SlashColor
        End Get

        Set(ByVal value As Color)
            _SlashColor = value
            DrawColorBox()
        End Set

    End Property

    ''' <summary>
    ''' ボックスの色
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("カラーボックス")> _
    <Description("ボックスを塗り潰す色を指定します。")> _
    <DefaultValue(GetType(Color), "Black")> _
    Public Property BoxColor() As Color

        Get
            Return _BoxColor
        End Get

        Set(ByVal value As Color)
            _BoxColor = value
            DrawColorBox()
        End Set

    End Property

    ''' <summary>
    ''' カラーボックスを作る
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MakeColorBox()

        Bmp = New Bitmap(ColorBoxSize.Width, ColorBoxSize.Height)
        Grp = Graphics.FromImage(Bmp)

        DrawColorBox()

    End Sub

    ''' <summary>
    ''' カラーボックスを描画する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DrawColorBox()

        With Grp
            .Clear(BoxColor)                                                           '背景を塗る。
            .DrawLine(New Pen(SlashColor), 0, 0, Bmp.Width - 1, Bmp.Height - 1)        '斜線を引く。
            .DrawRectangle(New Pen(Me.ForeColor), 0, 0, Bmp.Width - 1, Bmp.Height - 1) '枠を描く。
        End With

        Me.Image = Bmp '画像を再描画するたびに《.Image》プロパティに登録し直さないと、コントロールに反映されないみたい。《PictureBox.Image》とは違うところ。

        Me.Refresh()

    End Sub

End Class
