﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『[Keys]の内容のXMLシリアライズについて』
'  │・[Keys]に修飾子付きキーコードを設定してXML化すると、余計なものが出力される。
'  │例）[Keys.Alt Or Keys.Up] → "RButton MButton XButton2 Space Next PageDown Home Up Alt"（"RButton MButton XButton2 Space Next PageDown Home"が不用）
'  │・これは、キーコードの重複が原因だと思われる。このままでも、不用な部分を削除しても、キーコード自体は全く同じ。
'  │・このままでも、不用な部分を削除しても、キーコードの中身は全く同じ値なので、可読性は犠牲になるがそのままにしておく。
'  │
'  │『[Color]構造体のXMLシリアライズについて』
'  │・[Color]構造体は、XMLシリアライズに対応していません。《IXmlSerializable》インターフェイスを使えばXMLへの入出力をコントロールできるらしいが、いかんせんやり方が分からない。
'  │・そこで、色（名称またはARGB値）の入出力専用プロパティを用意して対処することにした。その代わり、読み込み時のエラーで形式を判断するため、デバッグモードで起動するとエラーメッセージが表示されてしまう（無視すればいい）。
'  │→ 単純に10進数で書き出されても、XMLの記述を直接書き換えにくいので、16進数でやり取りできる必要がある。その為には、色名の入出力とARGB値の16進数出力ができる《Color.Name》,《Color.FromName》は、非常に好都合だった。
'  │   多様な表現を無視するのであれば《ColorTranslator.FromHtml》&《ColorTranslator.ToHtml》メソッドを用いて単純化することも可能だろうけど、実験的な意味も込めた実装にしてみた。それに、HTML系への変換では《SystemColors》の一部で空白を返す問題がある。
'  │
'  │『プロパティの初期値の記述場所ついて』
'  │・プロパティの初期値は、定義箇所に直接的に記述している物と、インスタンス時に《With》で間接的に記述している物の、2種類がある。
'  │→ 直接記述では、XMLに当該項目が記述されていなくても、その値が適用される。記述時に、Intellisenseが働かないのが不便である。
'  │→ 間接記述では、上位クラスのインスタンシングでのみ値が適用される。デシリアライズでは空のままなおで、ロード時のエラーチェックに引っ掛けられる。
'  └────────────





''' <summary>
''' ファイル管理
''' </summary>
''' <remarks></remarks>
Public Module FileManager

    ''' <summary>設定ファイル名</summary>
    Public ReadOnly SettingFileName As String = System.IO.Path.GetFullPath(".\Setting.xml")

    ''' <summary>記録ファイル名</summary>
    Public ReadOnly ResumeFileName As String = System.IO.Path.GetFullPath(".\Resume.xml")


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>設定内容</summary>
    Public SettingData As New SettingParameter

    ''' <summary>記録内容</summary>
    Public ResumeData As New ResumeParameter


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


#Region "Load"

    ''' <summary>
    ''' XMLファイルからの復元
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="LoadObject">復元先</param>
    ''' <remarks></remarks>
    Private Sub FileLoadBase(FileName As String, ByRef LoadObject As Object)

        '=== XMLファイルから読み込む ===
        If System.IO.File.Exists(FileName) Then

            Try

                '--- XMLファイルから読み込み、逆シリアル化する ---
                Dim serializer As New System.Xml.Serialization.XmlSerializer(LoadObject.GetType) 'XMLシリアル化機構を作成する。
                Using streamer As New System.IO.FileStream(FileName, System.IO.FileMode.Open) 'ファイルを開く。
                    LoadObject = CTypeDynamic(serializer.Deserialize(streamer), LoadObject.GetType) 'ファイルから復元する。
                End Using

            Catch ex As Exception

                Throw 'エラーを再スロー

            End Try

        End If

    End Sub

    ''' <summary>
    ''' 設定内容を復元する
    ''' </summary>
    ''' <param name="LoadObject">読込先</param>
    ''' <returns>不正値修正結果</returns>
    ''' <remarks></remarks>
    Public Function SettingLoad(ByRef LoadObject As SettingParameter) As String

        Try
            FileLoadBase(SettingFileName, LoadObject)
        Catch ex As Exception
            Throw
        End Try

        Return InjusticeValueCheck() '不正値修正

    End Function

    ''' <summary>
    ''' 記録内容を復元する
    ''' </summary>
    ''' <param name="LoadObject">読込先</param>
    ''' <remarks></remarks>
    Public Sub ResumeLoad(ByRef LoadObject As ResumeParameter)

        Try
            FileLoadBase(ResumeFileName, LoadObject)
        Catch ex As Exception
            Throw
        End Try

    End Sub

    ''' <summary>
    ''' 不正値の確認と修正
    ''' </summary>
    ''' <remarks></remarks>
    Public Function InjusticeValueCheck() As String

        'ファイルに記述がない場合は、未設定（変数等の初期値）の状態で起動するため、未設定では不都合なものを修正する。
        'ちなみに、ファイルの記述が意味不明な場合は、XML解析の段階でエラーとなり、コードに記述したデフォルト値の状態で起動する。要するに、ファイルがなかった場合と同じ。

        '=== 設定値の修正 ===
        Dim errorItem As New List(Of String) '問題があった項目名
        Dim DefaultStteing As New SettingParameter 'デフォルト値

        With SettingData

            With .Lupe

                If .RefreshInterval < FormMain_Viewer.REFRESH_INTERVAL_MINIMUM OrElse .RefreshInterval > FormMain_Viewer.REFRESH_INTERVAL_MAXIMUM Then
                    .RefreshInterval = DefaultStteing.Lupe.RefreshInterval
                    errorItem.Add("範囲外：<拡大鏡>-<更新間隔>")
                End If

                With .Viewer

                    If .ViewSize.Width < FormMain_Viewer.VIEW_ADJUST_MINIMUM OrElse .ViewSize.Width > FormMain_Viewer.VIEW_ADJUST_MAXIMAM OrElse .ViewSize.Height < FormMain_Viewer.VIEW_ADJUST_MINIMUM OrElse .ViewSize.Height > FormMain_Viewer.VIEW_ADJUST_MAXIMAM Then
                        .ViewSize = DefaultStteing.Lupe.Viewer.ViewSize
                        errorItem.Add("範囲外：<拡大鏡>-<ビューア>-<寸法>")
                    End If

                    If .Magnification < FormMain_Viewer.MAGNIFICATION_MINIMUM OrElse .Magnification > FormMain_Viewer.MAGNIFICATION_MAXIMUM Then
                        .Magnification = FormMain_Viewer.MAGNIFICATION_MINIMUM
                        errorItem.Add("範囲外：<拡大鏡>-<ビューア>-<倍率>")
                    End If

                End With

                With .Avoid
                    If .ExerciseDistance < FormMain_Viewer.NEAR_DISTANCE_MINIMUM OrElse .ExerciseDistance > FormMain_Viewer.NEAR_DISTANCE_MAXIMUM Then
                        .ExerciseDistance = DefaultStteing.Lupe.Avoid.ExerciseDistance
                        errorItem.Add("範囲外：<拡大鏡>-<回避>-<発動距離>")
                    End If
                End With

                With .InformationPanel.Font
                    If .Size < FormInfoPanel_Base.FONT_SIZE_MINIMUM OrElse .Size > FormInfoPanel_Base.FONT_SIZE_MAXIMUM Then
                        .Size = DefaultStteing.Lupe.InformationPanel.Font.Size
                        errorItem.Add("範囲外：<拡大鏡>-<情報パネル>-<フォント>-<サイズ>")
                    End If
                End With

            End With

            With .CommandPad

                If .Opacity < 0.1 Then '許容範囲：10%～（《Opacity》は、範囲外の数値を入れてもエラーを出さない（自動的に0または1になる）ので対処を必要としないけど、透明では使えないので下限のみ設ける。）
                    .Opacity = DefaultStteing.CommandPad.Opacity
                    errorItem.Add("範囲外：<コマンドパッド>-<不透明度>")
                End If

            End With

        End With

        Return String.Join(vbCrLf, errorItem.ToArray)

    End Function

#End Region

#Region "Save"

    ''' <summary>
    ''' XMLファイルへの保存
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="SaveObject">保存元</param>
    ''' <remarks></remarks>
    Private Sub FileSaveBase(FileName As String, SaveObject As Object)

        '=== XMLファイルに書き込む ===
        Try

            '--- シリアル化し、XMLファイルに保存する ---
            Dim serializer As New System.Xml.Serialization.XmlSerializer(SaveObject.GetType) 'XMLシリアル化機構を作成する。
            Using streamer As New System.IO.FileStream(FileName, System.IO.FileMode.Create) 'ファイルを開く。
                serializer.Serialize(streamer, SaveObject) '用意したファイルに書き出す。
            End Using

        Catch ex As Exception

            Throw 'エラーを再スロー

        End Try

    End Sub

    ''' <summary>
    ''' 設定内容を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub SettingSave()

        Try
            FileSaveBase(SettingFileName, SettingData)
        Catch ex As Exception
            Throw
        End Try

    End Sub

    ''' <summary>
    ''' 記録内容を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ResumeSave()

        Try
            FileSaveBase(ResumeFileName, ResumeData)
        Catch ex As Exception
            Throw
        End Try

    End Sub

#End Region

End Module





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 設定項目
''' </summary>
''' <remarks></remarks>
<Serializable()> _
<System.Xml.Serialization.XmlRoot("HELuPERの設定内容")> _
Public Class SettingParameter

    ''' <summary>
    ''' 複製する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Clone() As SettingParameter

        'オブジェクトの内容を、メモリ上に一旦バイナリ形式でシリアル化し、再び逆シリアル化することで、複製してしまう。（参考：http://d.hatena.ne.jp/tekk/20100131/1264913887）
        '型を[Object]にすれば、いろんな形に対応できる。
        '速度的な利点はないかもしれないけど、抜群のお手軽さがある。特に、自作クラスなど、どんな複雑な物でも一発で簡単に複製できてしまう魔法のコード。全項目を一つずつコピーするなんて超面倒な作業にオサラバ。
        '（あまりにも入れ子状態が深すぎると駄目なのかも・・・http://support.microsoft.com/default.aspx?scid=kb;JA;320079）
        '※複製する全てのクラスの頭に、"<Serializable()>"を付けなければならない。

        Dim twinObject As SettingParameter '複製したオブジェクト

        Try

            Dim formatter As New System.Runtime.Serialization.Formatters.Binary.BinaryFormatter() 'バイナリ形式シリアル化機構を作成する。
            Using streamer As New System.IO.MemoryStream() 'メモリを用意する。
                formatter.Serialize(streamer, Me) 'メモリに書き出す。
                streamer.Position = 0                        '┬ メモリから復元する。
                twinObject = formatter.Deserialize(streamer) '┘
            End Using

            'Throw New Exception("テスト") 'テスト用にエラーを発生させてみる。

        Catch ex As Exception

            Throw New ApplicationException("設定内容の複製に失敗しました。" & vbCrLf & "（" & ex.Message & "）")

        End Try

        Return twinObject

    End Function



    '============================================================================================================================



    ''' <summary>
    ''' 拡大鏡に関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("拡大鏡")> _
    Public Property Lupe() As LupeParameter = New LupeParameter With {.RefreshInterval = 100}

    ''' <summary>
    ''' 操作キーに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("操作キー")> _
    Public Property OperationKey() As OperationKeyParameter = New OperationKeyParameter With {.ComPadCallKeys = Keys.Control Or Keys.Alt Or Keys.LWin}

    ''' <summary>
    ''' コマンドパッドに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("コマンドパッド")> _
    Public Property CommandPad() As New CommandPadParameter



    '============================================================================================================================



    ''' <summary>
    ''' 拡大鏡に関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class LupeParameter

        ''' <summary>
        ''' 更新する間隔を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("更新間隔")> _
        Public Property RefreshInterval() As Integer

        ''' <summary>
        ''' ビューアに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ビューア")> _
        Public Property Viewer() As ViewerParameter = New ViewerParameter With {.ViewSize = New Size(120, 120), .Magnification = 8}

        ''' <summary>
        ''' マウスカーソル回避に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("回避")> _
        Public Property Avoid() As AvoidParameter = New AvoidParameter With {.Type = AvoidActionTypes.RightTriangle, .ExerciseDistance = 10}

        ''' <summary>
        ''' 情報パネルに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("情報パネル")> _
        Public Property InformationPanel() As InformationPanelParameter = New InformationPanelParameter With {.Style = InfoPanelStyles.Bar}



        '============================================================================================================================



        ''' <summary>
        ''' ビューアに関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class ViewerParameter

            ''' <summary>
            ''' ビューの寸法を設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("寸法")> _
            Public Property ViewSize() As Size

            ''' <summary>
            ''' 倍率を設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("倍率")> _
            Public Property Magnification() As Integer

            ''' <summary>
            ''' マーク
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("マーク")> _
            Public Property Mark() As Switch1Color2 = New Switch1Color2 With {.Enabled = True, .MainColor = Color.Red, .SubColor = Color.FromArgb(&HAA3F3F3F)}

            ''' <summary>
            ''' サイト
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("サイト")> _
            Public Property Sight() As Switch1Color1 = New Switch1Color1 With {.Enabled = True, .MainColor = Color.Orange}

            ''' <summary>
            ''' サイトX
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("サイトX")> _
            Public Property SightX() As Boolean = True

            ''' <summary>
            ''' 格子線
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("格子線")> _
            Public Property Grid() As Switch1Color1 = New Switch1Color1 With {.Enabled = True, .MainColor = Color.Black}

            ''' <summary>
            ''' 下地
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("下地")> _
            Public Property Foundation() As Switch1Color2 = New Switch1Color2 With {.Enabled = True, .MainColor = Color.Black, .SubColor = Color.White}

            ''' <summary>
            ''' 下地の模様
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("下地模様")> _
            Public Property Pattern() As Drawing2D.HatchStyle = Drawing2D.HatchStyle.ForwardDiagonal



            '============================================================================================================================



            ''' <summary>
            ''' スイッチ付きの色
            ''' </summary>
            ''' <remarks>1スイッチ/1色</remarks>
            <Serializable()> _
            Public Class Switch1Color1

                ''' <summary>
                ''' 有効/無効
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("有効")> _
                Public Property Enabled() As Boolean

                ''' <summary>
                ''' 主色
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks>《Color》構造体をシリアル化する方法が分からないのと、16進数ARGB値や色名の方がXMLを直接書き換え易いので、別途《MainColorCode》を用意する。</remarks>
                <System.Xml.Serialization.XmlIgnoreAttribute()> _
                Public Property MainColor() As Color

                ''' <summary>
                ''' 主色のコード
                ''' </summary>
                ''' <value>色名（大文字小文字不問）または16進数ARGB値（"0x"付きも可）</value>
                ''' <returns>色名またはARGB値（16進数）</returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("主色")> _
                <System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never), System.ComponentModel.Browsable(False)> _
                Public Property MainColorCode() As String

                    Get
                        Return ColorExport(MainColor)
                    End Get

                    Set(ByVal value As String)
                        MainColor = ColorImport(value)
                    End Set

                End Property



                '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



                ''' <summary>
                ''' 色を文字列に変換する
                ''' </summary>
                ''' <param name="Value">《Color》構造体</param>
                ''' <returns>文字列</returns>
                ''' <remarks></remarks>
                Protected Function ColorExport(Value As Color) As String

                    Return Value.Name '名前がある場合は色名が、ない場合はARGB値（16進数）が返される。便利。
                    'Return Convert.ToString(Value.ToArgb, 16) 'これだと16進数のみ。

                End Function

                ''' <summary>
                ''' 文字列を色に変換する
                ''' </summary>
                ''' <param name="Value">文字列</param>
                ''' <returns>《Color》構造体</returns>
                ''' <remarks></remarks>
                Protected Function ColorImport(Value As String) As Color

                    Dim ret As Color

                    Try
                        If Value(0) = "#"c Then
                            ret = ColorTranslator.FromHtml(Value) 'HTML形式(#aarrggbb or #rrggbb)に対応。※上位桁の0を省略しない→#00ff00。
                        Else
                            ret = Color.FromArgb(Convert.ToInt32(Value, 16)) '《Convert.ToInt32(～,16)》は、"0x"付きでも変換可能。
                        End If
                    Catch ex As Exception
                        ret = Color.FromName(Value) '16進数として認識できない場合は、色名と捉える。《Color.FromName(～)》には例外が無く、色名として有効でない場合は、透明（ARGB全て0）となる。
                    End Try
                    'Dim colorCode As Integer
                    'If Integer.TryParse(Value, Globalization.NumberStyles.HexNumber, Globalization.CultureInfo.CurrentCulture, colorCode) Then '《Integer.TryParse》なら《Try》構文を使わなくてもいい。その代わり、"0x"が付けられない。
                    '    ret = Color.FromArgb(colorCode)
                    'Else
                    '    ret = Color.FromName(Value)
                    'End If

                    Return ret

                End Function

            End Class



            '============================================================================================================================



            ''' <summary>
            ''' スイッチ付きの色
            ''' </summary>
            ''' <remarks>1スイッチ/2色</remarks>
            <Serializable()> _
            Public Class Switch1Color2

                Inherits Switch1Color1

                '----------------------------------------------------------------------------------------------------------------------------

                ''' <summary>
                ''' 副色
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks>《Color》構造体をシリアル化する方法が分からないのと、16進数ARGB値や色名の方がXMLを直接書き換え易いので、別途《SubColorCode》を用意する。</remarks>
                <System.Xml.Serialization.XmlIgnoreAttribute()> _
                Public Property SubColor() As Color

                ''' <summary>
                ''' 副色のコード
                ''' </summary>
                ''' <value>色名（大文字小文字不問）または16進数ARGB値（"0x"付きも可）</value>
                ''' <returns>色名またはARGB値（16進数）</returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("副色")> _
                <System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never), System.ComponentModel.Browsable(False)> _
                Public Property SubColorCode() As String

                    Get
                        Return ColorExport(SubColor)
                    End Get

                    Set(ByVal value As String)
                        SubColor = ColorImport(value)
                    End Set

                End Property

            End Class

        End Class



        '============================================================================================================================



        ''' <summary>
        ''' マウスカーソル回避に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class AvoidParameter

            ''' <summary>
            ''' 回避方法を設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("方法")> _
            Public Property Type() As AvoidActionTypes

            ''' <summary>
            ''' 回避を発動する（マウスの接近を許す）距離を設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks>負数になったり《Integer》型の最大値を超えたりしたら困るし、それほど大きい数字も必要ないから《UShort》型</remarks>
            <System.Xml.Serialization.XmlElement("発動距離")> _
            Public Property ExerciseDistance() As UShort

            ''' <summary>
            ''' 画面間をまたぐかを設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("越境")> _
            Public Property Cross() As Boolean = True

        End Class



        '============================================================================================================================



        ''' <summary>
        ''' 情報パネルに関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class InformationPanelParameter

            ''' <summary>
            ''' 形式
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("形式")> _
            Public Property Style() As InfoPanelStyles

            ''' <summary>
            ''' 
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("フォント")> _
            Public Property Font() As New FontParameter



            '============================================================================================================================



            ''' <summary>
            ''' フォントに関する設定項目
            ''' </summary>
            ''' <remarks></remarks>
            <Serializable()> _
            Public Class FontParameter

                ''' <summary>
                ''' 名称
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("名称")> _
                Public Property Name() As String = "ＭＳ ゴシック"

                ''' <summary>
                ''' スタイル
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("スタイル")> _
                Public Property Style() As FontStyle = FontStyle.Regular

                ''' <summary>
                ''' サイズ
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("サイズ")> _
                Public Property Size() As Single = 9.0F



                '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



                ''' <summary>
                ''' フォントを格納する
                ''' </summary>
                ''' <param name="NewFont">新しいフォント</param>
                ''' <remarks></remarks>
                Public Sub SetFont(NewFont As Font)

                    With NewFont
                        Name = .Name
                        Style = .Style
                        Size = .Size
                    End With

                End Sub

                ''' <summary>
                ''' 現在設定されているフォントを得る
                ''' </summary>
                ''' <returns></returns>
                ''' <remarks></remarks>
                Public Function GetFont() As Font

                    Return New Font(Name, Size, Style)

                End Function

            End Class

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' 操作キーに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class OperationKeyParameter

        ''' <summary>
        ''' 指呼キーを有効にするかを設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("コマンドパッド指呼キー有効")> _
        Public Property ComPadCallKeyEnabled As Boolean = True

        ''' <summary>
        ''' ダイレクトキーを有効にするかを設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ダイレクトキー有効")> _
        Public Property DirectKeyEnabled As Boolean = True

        ''' <summary>
        ''' コマンドパッド指呼キー
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks>ダイレクトに機能を利用するキーではないので、操作キーではあるけどダイレクトキーではない。</remarks>
        <System.Xml.Serialization.XmlElement("コマンドパッド指呼キー")> _
        Public Property ComPadCallKeys As Keys

        ''' <summary>
        ''' ダイレクトキーに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks>操作キーの内のダイレクトキー</remarks>
        <System.Xml.Serialization.XmlElement("ダイレクトキー")> _
        Public Property DirectKeys() As DirectKeyParameter = New DirectKeyParameter With {.CursorUpKey = Keys.Control Or Keys.Up,
                                                                                          .CursorDownKey = Keys.Control Or Keys.Down,
                                                                                          .CursorLeftKey = Keys.Control Or Keys.Left,
                                                                                          .CursorRightKey = Keys.Control Or Keys.Right,
                                                                                          .CaptureKey = Keys.Shift Or Keys.Control Or Keys.C}

        ''' <summary>
        ''' [Shift]キーを押した時の不透明度を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("Shift不透明度")> _
        Public Property ShiftOpacity As Double = 0.0R

        ''' <summary>
        ''' [Ctrl]キーを押した時の不透明度を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("Ctrl不透明度")> _
        Public Property ControlOpacity As Double = 1.0R

        ''' <summary>
        ''' [Alt]キーを押した時の不透明度を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("Alt不透明度")> _
        Public Property AltOpacity As Double = 1.0R



        '============================================================================================================================



        ''' <summary>
        ''' ダイレクトキーに関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class DirectKeyParameter

            ''' <summary>
            ''' カーソル上キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル上")> _
            Public Property CursorUpKey As Keys

            ''' <summary>
            ''' カーソル下キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル下")> _
            Public Property CursorDownKey As Keys

            ''' <summary>
            ''' カーソル左キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル左")> _
            Public Property CursorLeftKey As Keys

            ''' <summary>
            ''' カーソル右キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル右")> _
            Public Property CursorRightKey As Keys

            ''' <summary>
            ''' キャプチャ キー
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("キャプチャ")> _
            Public Property CaptureKey As Keys

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' コマンドパッドに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class CommandPadParameter

        ''' <summary>
        ''' コマンドパッドの不透明度
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("不透明度")> _
        Public Property Opacity As Double = 1.0R

        ''' <summary>
        ''' 色取得用書式に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("色書式")> _
        Public Property ColorFormat As ColorFormatParameter = New ColorFormatParameter With {.Ready = New String() {"{3},{4},{5},{6}", "{3:X2},{4:X2},{5:X2},{6:X2}", "{1:X8}", "{2:X6}", "{6:X2}{5:X2}{4:X2}", "rgba({4}, {5}, {6}, {7:0.0})"},
                                                                                             .Using = "{1:X8}"}

        <System.Xml.Serialization.XmlElement("送出パターン")> _
        Public Property SendPattern As String = "^a|RED|{TAB}^a|GREEN|{TAB}^a|BLUE|+{TAB 2}"



        '============================================================================================================================



        ''' <summary>
        ''' 色書式に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class ColorFormatParameter

            ''' <summary>
            ''' 既製色書式
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("既製")> _
            Public Property Ready As String()

            ''' <summary>
            ''' 使用する色書式
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("使用")> _
            Public Property [Using] As String

        End Class

    End Class

End Class





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 記録項目
''' </summary>
''' <remarks></remarks>
<System.Xml.Serialization.XmlRoot("HELuPERの記録内容")> _
Public Class ResumeParameter

    ''' <summary>
    ''' ビューアの表示位置（フォームの中心座標）
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>マウスからの回避モードが、〔手取り足取り〕の時に使われる</remarks>
    <System.Xml.Serialization.XmlElement("ビューア位置")> _
    Public Property ViewerLocate() As Point = New Point(0, 0)

    ''' <summary>
    ''' 基点
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("基点")> _
    Public Property CardinalPoint() As Point = New Point(0, 0)



    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



    Private memoLocate As Point        '表示位置の記憶用
    Private memoCardinalPoint As Point '基点の記憶用


    '----------------------------------------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 現在値を記憶しておく
    ''' </summary>
    ''' <remarks>変更有無の比較用</remarks>
    Public Sub Memo()

        memoLocate = ViewerLocate
        memoCardinalPoint = CardinalPoint

    End Sub

    ''' <summary>
    ''' 記録項目に変更があるか調べる
    ''' </summary>
    ''' <returns>変更があれば[True]、なければ[False]を返す</returns>
    ''' <remarks></remarks>
    Public Function Modified() As Boolean

        Return (memoLocate <> ViewerLocate OrElse memoCardinalPoint <> CardinalPoint)

    End Function

End Class