﻿''' <summary>
''' コマンドパッド：金平糖（大玉）
''' </summary>
''' <remarks>基底クラスに拡張フォームの《UnProtrudeForm》を使用する。</remarks>
Public Class FormCommandPad

    ''' <summary>
    ''' コマンドイベント
    ''' </summary>
    ''' <param name="sender">発生元</param>
    ''' <param name="e">要求内容</param>
    ''' <remarks></remarks>
    Public Event Action(sender As Object, e As CommandEventArgs)



    '===========================================================================================================



    Private Const WS_EX_NOACTIVATE As Integer = &H8000000 '非アクティブのスタイルを指定する値

    Protected Overrides ReadOnly Property CreateParams() As CreateParams

        Get
            '=== 非アクティブ化 ===
            'Microsoftによると、これだけでアクティブ化しないフォームになるみた。
            '《.ExStyle》プロパティは、“拡張ウィンドウスタイル”と呼ばれるもので、ウィンドウのいろいろな状態を設定できるみたい。
            '“拡張ウィンドウスタイル”で設定できる項目の内、「アクティブにならないようにする」ためのフラグが、《WS_EX_NOACTIVATE(&H8000000)》ということですな。

            Dim p As CreateParams = MyBase.CreateParams
            If Not MyBase.DesignMode Then
                p.ExStyle = p.ExStyle Or (WS_EX_NOACTIVATE)
            End If
            Return p
        End Get

    End Property



    '===========================================================================================================



    ''' <summary>注目点の調整幅</summary>
    ''' <remarks>最小値には、これを負数にしたものを用いる。</remarks>
    Private Const OffsetRange As Integer = 100



    '===========================================================================================================



    Private ConsoleArea As Rectangle 'コンソール（ボタン類がある）領域


    '-----------------------------------------------------------------------------------------------------------


    Private Sub FormCommandPad_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load

        '=== コマンドパッドの要請内容を設定 ===
        btnSetCardinalPoint.Tag = CommandEventArgs.Requests.SetCardinalPoint
        btnGotoCardinalPoint.Tag = CommandEventArgs.Requests.GotoCardinalPoint
        btnGotoHome.Tag = CommandEventArgs.Requests.GotoHome
        btnGetColor.Tag = CommandEventArgs.Requests.GetColor
        btnSendRGB.Tag = CommandEventArgs.Requests.SendRGB
        btnCopyView.Tag = CommandEventArgs.Requests.CopyImage

        '=== コンソール領域を抽出 ===
        '全コントロールの形状を合成することで、コンソール領域を得る。
        ConsoleArea.Location = Me.Controls(0).Location '左上隅の座標がコンソール領域外だとまずいので、適当な座標を入れておく。
        For Each item As Control In Me.Controls
            ConsoleArea = Rectangle.Union(ConsoleArea, item.Bounds)
        Next
        ConsoleArea.Inflate(1, 1) '輪郭用に太らす。

        '=== 注目点調整範囲設定 ===
        nudOffsetX.Minimum = -OffsetRange
        nudOffsetX.Maximum = OffsetRange
        nudOffsetY.Minimum = nudOffsetX.Minimum
        nudOffsetY.Maximum = nudOffsetX.Maximum

    End Sub

    Private Sub FormCommandPad_FormClosing(sender As Object, e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        '=== ユーザ操作による終了（例えば、[Alt]+[F4]）を却下 ===
        If e.CloseReason = CloseReason.UserClosing Then
            e.Cancel = True
        End If

    End Sub

    Private Sub FormCommandPad_VisibleChanged(sender As Object, e As System.EventArgs) Handles Me.VisibleChanged

        '=== 表示/非表示の切り替え ===
        If Me.Visible Then

            '--- 準備 ---
            Const TOP_LEFT As Integer = 0, TOP_RIGHT As Integer = 1, BOTTOM_LEFT As Integer = 2, BOTTOM_RIGHT As Integer = 3 '位置取りを示す番号
            Const PROJECTION_SIZE As Integer = 8 '標準的な突起のサイズ
            Const JOINT As Integer = 16 '節を作る場所（四隅からの距離）

            Dim target As Point = DirectCast(Owner, FormMain_Viewer).ShootingLocation '注目点
            Dim nowScreen As Rectangle = Screen.GetWorkingArea(target) '注目点がある作業領域

            Dim Positioning As Integer = If(target.X < nowScreen.Left + nowScreen.Width \ 2, TOP_LEFT, TOP_RIGHT) + If(target.Y < nowScreen.Top + nowScreen.Height \ 2, 0, BOTTOM_LEFT) '画面を4分割した時の位置

            nowScreen.Inflate(-4, -4) 'タスクバーとの間に余白を設ける。別になくても構わない。

            '--- 表示位置 ---
            Dim newLocatin As Point = target '新しい表示先

            Select Case Positioning '注目点に対する位置取りと、突起を出すスペースを空ける。
                Case TOP_LEFT
                    newLocatin += New Size(PROJECTION_SIZE, PROJECTION_SIZE)
                Case TOP_RIGHT
                    newLocatin += New Size(-PROJECTION_SIZE - ConsoleArea.Width, PROJECTION_SIZE)
                Case BOTTOM_LEFT
                    newLocatin += New Size(PROJECTION_SIZE, -PROJECTION_SIZE - ConsoleArea.Height)
                Case BOTTOM_RIGHT
                    newLocatin += New Size(-PROJECTION_SIZE - ConsoleArea.Width, -PROJECTION_SIZE - ConsoleArea.Height)
            End Select

            newLocatin = UnProtrude(newLocatin, ConsoleArea.Size, nowScreen) '作業領域に全ボタンが収まるように調整する。
            Me.Location = newLocatin - New Size(ConsoleArea.Left, ConsoleArea.Top) 'ボタン領域の左上が基準表示位置に来るように、フォームの表示位置をずらす。

            '--- 形状 ---
            target = Me.PointToClient(target) '注目点をフォーム上のクライアント座標に変換する。

            Using outline As New Drawing2D.GraphicsPath '輪郭
                With ConsoleArea
                    outline.AddLines(
                        New Point() {
                            New Point(.Left, .Top + JOINT),
                            If(Positioning = TOP_LEFT, target, .Location),
                            New Point(.Left + JOINT, .Top),
                            New Point(.Right - JOINT, .Top),
                            If(Positioning = TOP_RIGHT, target, New Point(.Right, .Top)),
                            New Point(.Right, .Top + JOINT),
                            New Point(.Right, .Bottom - JOINT),
                            If(Positioning = BOTTOM_RIGHT, target, New Point(.Right, .Bottom)),
                            New Point(.Right - JOINT, .Bottom),
                            New Point(.Left + JOINT, .Bottom),
                            If(Positioning = BOTTOM_LEFT, target, New Point(.Left, .Bottom)),
                            New Point(.Left, .Bottom - JOINT)
                        }
                    )
                End With
                Me.Region = New Region(outline)
            End Using

            '--- 不透明度 ---
            Me.Opacity = SettingData.CommandPad.Opacity

            '--- 各機能の準備 ---
            With SettingData.CommandPad
                btnSendRGB.Enabled = (.SendPattern <> "")
                btnGetColor.Enabled = (.ColorFormat.Using <> "")
            End With
            btnSendRGB.BackColor = Me.BackColor
            nudOffsetX.Value = 0
            nudOffsetY.Value = 0

            '--- マウスカーソル移動範囲規制 ---
            'コマンドパッドを表示しながら他画面に出て強制サイズ調整が発生すると、拡大画像が何にもなしになっちゃう問題を回避する。そもそも、コマンドパッドを使用中に外へ出る必要もない。
            Windows.Forms.Cursor.Clip = Me.RectangleToScreen(ConsoleArea)

            '--- マウスカーソルをホームボタン位置へ ---
            With btnGotoCardinalPoint
                Windows.Forms.Cursor.Position = .PointToScreen(New Point(.Width \ 2, .Height \ 2))
            End With

        Else

            '--- マウスカーソルを元の位置へ ---
            Windows.Forms.Cursor.Clip = Nothing '移動範囲解除
            Windows.Forms.Cursor.Position = DirectCast(Owner, FormMain_Viewer).ShootingLocation + New Point(nudOffsetX.Value, nudOffsetY.Value)

            '--- RGB送出イベント発生 ---
            If btnSendRGB.BackColor <> Me.BackColor Then
                RaiseEvent Action(Me, New CommandEventArgs(btnSendRGB.Tag))
            End If

        End If

    End Sub

    Private Sub CommandButton_MouseEnter(sender As Object, e As System.EventArgs) Handles btnSetCardinalPoint.MouseEnter, btnGotoCardinalPoint.MouseEnter, btnGotoHome.MouseEnter, btnGetColor.MouseEnter, btnCopyView.MouseEnter, btnSendRGB.MouseEnter

        '=== 選択強調表示 ===
        With DirectCast(sender, Button)
            Dim bmp As New Bitmap(.ClientSize.Width, .ClientSize.Height)
            Using gBrush As New Drawing2D.LinearGradientBrush(.ClientRectangle, Color.FromArgb(&H8, SystemColors.MenuHighlight), Color.FromArgb(&H32, SystemColors.MenuHighlight), Drawing2D.LinearGradientMode.Vertical) 'アルファ値は、適当（画像に重ねるので、《Button》コントロール等に比べれば濃い目）。
                Graphics.FromImage(bmp).FillRectangle(gBrush, .ClientRectangle)
                .Image = bmp
            End Using
        End With

        '=== ツールチップ更新 ===
        If sender Is btnGotoCardinalPoint Then ttpHint.SetToolTip(btnGotoCardinalPoint, String.Format("基点({0,5:D4},{1,5:D4})へ移動します。", ResumeData.CardinalPoint.X, ResumeData.CardinalPoint.Y))

    End Sub

    Private Sub CommandButton_MouseLeave(sender As Object, e As System.EventArgs) Handles btnSetCardinalPoint.MouseLeave, btnGotoCardinalPoint.MouseLeave, btnGotoHome.MouseLeave, btnGetColor.MouseLeave, btnCopyView.MouseLeave, btnSendRGB.MouseLeave

        '=== 選択強調解除 ===
        With DirectCast(sender, Button)
            If .Image IsNot Nothing Then
                .Image.Dispose()
                .Image = Nothing
            End If
        End With

    End Sub

    Private Sub CommandButton_Click(sender As System.Object, e As System.EventArgs) Handles btnSetCardinalPoint.Click, btnGotoCardinalPoint.Click, btnGotoHome.Click, btnGetColor.Click, btnCopyView.Click

        '=== コマンドイベント発生 ===
        RaiseEvent Action(Me, New CommandEventArgs(DirectCast(sender, Button).Tag))

    End Sub

    Private Sub btnSendRGB_Click(sender As System.Object, e As System.EventArgs) Handles btnSendRGB.Click

        '=== RGB送出の待機/解除 ===
        btnSendRGB.BackColor = If(btnSendRGB.BackColor = Me.BackColor, Color.Orange, Me.BackColor)

    End Sub

    Private Sub nudOffsetXY_ValueChanged(sender As Object, e As System.EventArgs) Handles nudOffsetX.ValueChanged, nudOffsetY.ValueChanged

        '=== 有無効状態の表示 ===
        With DirectCast(sender, NumericUpDown)
            .BackColor = If(.Value = 0, SystemColors.Control, Color.Orange)
        End With

    End Sub

    Private Sub btnOffsetCancel_Click(sender As System.Object, e As System.EventArgs) Handles btnOffsetCancel.Click

        '=== 調整値の取り消し ===
        nudOffsetX.Value = 0
        nudOffsetY.Value = 0

    End Sub

    Private Sub FormCommandPad_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint

        '=== 背景描画 ===
        '周りの色に埋もれず見えるように、明るい色と暗い色の2色で市松模様にする。
        e.Graphics.FillRectangle(New Drawing2D.HatchBrush(Drawing2D.HatchStyle.Percent50, SystemColors.ControlLightLight, SystemColors.ControlDark), Me.ClientRectangle)

    End Sub

End Class





'|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' コマンドイベント内容
''' </summary>
''' <remarks></remarks>
Public Class CommandEventArgs

    Inherits EventArgs



    '===========================================================================================================



    ''' <summary>
    ''' 要請一覧
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum Requests As Byte

        ''' <summary>未指定</summary>
        [Nothing]

        ''' <summary>基点登録</summary>
        SetCardinalPoint

        ''' <summary>基点一足飛</summary>
        GotoCardinalPoint

        ''' <summary>起点直行</summary>
        GotoHome

        ''' <summary>色取得</summary>
        GetColor

        ''' <summary>RGB送出</summary>
        SendRGB

        ''' <summary>拡大画像複写</summary>
        CopyImage

    End Enum



    '===========================================================================================================



    Private _Request As Requests

    ''' <summary>
    ''' 要請内容
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Request As Requests

        Get
            Return _Request
        End Get

    End Property



    '===========================================================================================================



    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="Request">要請内容</param>
    ''' <remarks></remarks>
    Public Sub New(Request As Requests)

        _Request = Request

    End Sub

End Class