﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『キーの押下とAPI《GetAsyncKeyState》の返り値の遷移』
'  │                                 ┌一発目    ┌中休み        ┌リピート有効      ┌名残り（次回APIが呼ばれるまで残る）
'  │ 何れかのキーを・・・│押す前 → │押した → │押し続けた → │更に押し続けた → │放した →→→→→→→→→→→→→→→ │押していない
'  │    APIの返り値・・・│&H0000 → │&H8001 → │&H8000 →→→ │&H8001 →→→→→ │&H0001 →→→→→→→→→→→→→→→ │&H0000
'  │《PressStatus》・・・└Off       └Valid     └On            └Valid             └Last                                  └Off
'  │※それなりに高頻度にスキャンしないと、キーリピート機能を有効に検知できない。
'  └────────────





'[FormMain_Viewer]クラスの続き
'操作キーに係わるルーチンをまとまる。

Imports System.Collections.Specialized

Partial Class FormMain_Viewer

    ''' <summary>
    ''' [API] キーボード スキャン
    ''' </summary>
    ''' <param name="vKey">調べたいキーのコード</param>
    ''' <returns>現在押されていたら最上位bit(16bit目)が立つ。前回の呼び出し以降に押されていたら最下位bitが立つ（こちらは、キーリピート機能に対応）。押されていなかったら0が返される。</returns>
    ''' <remarks></remarks>
    Declare Function GetAsyncKeyState Lib "user32.dll" (ByVal vKey As Integer) As Short


    '-----------------------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 押下状態
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum PressStatus As Short

        ''' <summary>押されていない</summary>
        Off = &H0S

        ''' <summary>押されている</summary>
        [On] = &H8000S

        ''' <summary>有効（キーリピート機能）</summary>
        Valid = &H8001S

        ''' <summary>押されていた</summary>
        Last = &H1S

    End Enum



    '===========================================================================================================



    Private Sub tmrOpeKeyScan_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrOpeKeyScan.Tick

        '=== 操作キー ===
        With SettingData.OperationKey
            If .ComPadCallKeyEnabled Then CallKey()
            If .DirectKeyEnabled Then DirectKey()
        End With

        '=== 一時透過 ===
        '複数のキーを同時に押している時は、透過しない。

        Dim opacityValue As Double '不透明度

        Dim push As New BitVector32(0) 'キー押下フラグ
        Dim b1 As Integer = BitVector32.CreateMask,
            b2 As Integer = BitVector32.CreateMask(b1),
            b3 As Integer = BitVector32.CreateMask(b2) 'ビットマスク

        With My.Computer.Keyboard
            push(b1) = .ShiftKeyDown
            push(b2) = .CtrlKeyDown
            push(b3) = .AltKeyDown
        End With

        Select Case push.Data
            Case b1
                opacityValue = SettingData.OperationKey.ShiftOpacity
            Case b2
                opacityValue = SettingData.OperationKey.ControlOpacity
            Case b3
                opacityValue = SettingData.OperationKey.AltOpacity
            Case Else
                opacityValue = 1.0R
        End Select
        If Me.Opacity <> opacityValue Then Me.Opacity = opacityValue

    End Sub



    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



    ''' <summary>
    ''' コマンドパッド指呼キー
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CallKey()

        If (KeyScan(SettingData.OperationKey.ComPadCallKeys) And PressStatus.On) = PressStatus.On Then
            If Not frmCommandPad.Visible Then frmCommandPad.Visible = True '出現
        Else
            If frmCommandPad.Visible Then frmCommandPad.Visible = False '消失
        End If

    End Sub

    ''' <summary>
    ''' ダイレクトキーの操作
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DirectKey()

        With SettingData.OperationKey.DirectKeys

            '=== ビューをクリップボードへキャプチャする ===
            If KeyScan(.CaptureKey) = PressStatus.Valid Then

                Try
                    Using tempBMP As New Bitmap(picView.Width, picView.Height)
                        picView.DrawToBitmap(tempBMP, picView.Bounds) 'コントロールのレンダリングをビットマップに対して行わせる。
                        Clipboard.SetImage(tempBMP)
                    End Using
                    Sound(SIGNAL_COMPLETE) '“手応え”として音を出す。
                Catch ex As Exception
                    MessageBox.Show("ビューのキャプチャに失敗しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                End Try

                Exit Sub '※※※ダイレクトキーは、一度に一機能しか使用しない。

            End If

            '=== マウスカーソル移動（斜め・キーリピート対応） ===
            '斜め入力には、2つのキー（ここでは修飾キーを含まない）を同時に押す必要があるが、2つ目のキーを押すと1つ目のキーの再開ピットが下がってしまう。
            'キーリピート機能を利用しながら、2つ目のキーを放して1つ目のキーを継続する場合、最上位ビット＆最下位ビットと斜め入力後を示すフラグを組み合わせて、状態を把握する必要がある。

            Static DoubleKeyFlag As Boolean = False '2キー入力済みフラグ
            Dim x, y As Integer '移動量

            '--- カーソルキーの状態を取得 ---
            Dim up As Short = KeyScan(.CursorUpKey)
            Dim down As Short = KeyScan(.CursorDownKey)
            Dim left As Short = KeyScan(.CursorLeftKey)
            Dim right As Short = KeyScan(.CursorRightKey)

            'On（最上位ビット）の状態
            Dim upOn As Boolean = ((up And PressStatus.On) = PressStatus.On)
            Dim downOn As Boolean = ((down And PressStatus.On) = PressStatus.On)
            Dim leftOn As Boolean = ((left And PressStatus.On) = PressStatus.On)
            Dim rightOn As Boolean = ((right And PressStatus.On) = PressStatus.On)

            'Last（最下位ビット）の状態
            Dim upLast As Boolean = ((up And PressStatus.Last) = PressStatus.Last)
            Dim downLast As Boolean = ((down And PressStatus.Last) = PressStatus.Last)
            Dim leftLast As Boolean = ((left And PressStatus.Last) = PressStatus.Last)
            Dim rightLast As Boolean = ((right And PressStatus.Last) = PressStatus.Last)

            '--- 方向判定 ---
            If upOn AndAlso leftOn Then '左上
                If upLast OrElse leftLast Then
                    x = -1
                    y = -1
                    DoubleKeyFlag = True
                End If
            ElseIf upOn AndAlso rightOn Then '右上
                If upLast OrElse rightLast Then
                    x = 1
                    y = -1
                    DoubleKeyFlag = True
                End If
            ElseIf downOn AndAlso leftOn Then '左下
                If downLast OrElse leftLast Then
                    x = -1
                    y = 1
                    DoubleKeyFlag = True
                End If
            ElseIf downOn AndAlso rightOn Then '右下
                If downLast OrElse rightLast Then
                    x = 1
                    y = 1
                    DoubleKeyFlag = True
                End If
            ElseIf upOn AndAlso (upLast OrElse DoubleKeyFlag) Then '上
                y = -1
            ElseIf downOn AndAlso (downLast OrElse DoubleKeyFlag) Then '下
                y = 1
            ElseIf leftOn AndAlso (leftLast OrElse DoubleKeyFlag) Then '左
                x = -1
            ElseIf rightOn AndAlso (rightLast OrElse DoubleKeyFlag) Then '右
                x = 1
            End If

            '--- キーが解除されたらフラグを下す ---
            If DoubleKeyFlag AndAlso (up Or down Or left Or right) = PressStatus.Off Then
                DoubleKeyFlag = False
            End If

            '--- 移動 ---
            If x <> 0 OrElse y <> 0 Then Windows.Forms.Cursor.Position += New Point(x, y)

        End With

    End Sub

    ''' <summary>
    ''' キーが押されているかを調べる
    ''' </summary>
    ''' <param name="TargetKeySet">調べるキーセット</param>
    ''' <returns>押下状態</returns>
    ''' <remarks></remarks>
    Public Function KeyScan(TargetKeySet As Keys) As PressStatus

        Dim state As Keys = Keys.None '押下状態
        Dim apiReturn As PressStatus '[GetAsyncKeyState]APIの戻り値

        Dim pureKey As Keys = TargetKeySet And Keys.KeyCode 'キーセットから純粋なキーを抽出する（修飾子キーを取り除く）。
        Dim scanKey As Keys(,) = {{Keys.ShiftKey, Keys.ControlKey, Keys.Menu, pureKey},
                                  {Keys.Shift, Keys.Control, Keys.Alt, pureKey}} '(0,n)=[GetAsyncKeyState]APIに渡すキーコード / (1,n)=調べるキーコード

        For i As Byte = 0 To 3 '調べるキーの数 = 修飾子キー3種 + 純粋キー1種

            '※重要！！
            '[PressStatus.Last]ビットは、[GetAsyncKeyState]APIでスキャンするまで残っている。
            'その影響を次回に持ち越さないために、目的のキーが押されていないと判明したとしても、最後までループを続ける必要がある。

            apiReturn = GetAsyncKeyState(scanKey(0, i))

            If apiReturn <> PressStatus.Off Then state = state Or scanKey(1, i)

        Next

        '=== 結果を返す ===
        Return If(TargetKeySet <> state, PressStatus.Off, apiReturn)

    End Function

End Class
