﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『画像の拡大について』
'  │・便利な標準機能の《Graphics.DrawImage》メソッドを使ってみたら…
'  │●問題点1・・・単純にこの機能を使って拡大しても、色の補間処理が行われるのでルーペにならない。《ScaleTransform》でワールド変換しても同じ結果。
'  │●問題点2・・・《Drawing2D.InterpolationMode.NearestNeighbor》指定で補間処理を無効化しても、本当に補間処理をすっ飛ばすだけで、補間に使用するスペースを空けた状態で拡大されてしまう。やっぱり、ルーペにならない。
'  │├→ しかたがないので、素直に“元画像から1ドットずつ色を読み取り、拡大先に四角を描画する”方法に落ち着いた。昔に比べればPCの能力は格段に向上しており、今回は拡大する範囲も限定的なので、この方法でも他の処理に大きな影響を与えずに済むと思う。
'  │└→ CPUに対して大した負荷ではないとはいえ、念のための更なる負荷軽減策として、拡大エリアと同じサイズの配列を用意してピクセル色を記憶しておき、前回と色が違う部分のみを描き直すようにしている。グラフィックメソッドより配列操作＆比較の方が処理が軽い場合に、簡単で有効な方法だと思う。
'  │
'  │『記録ファイル（"Resume.xml"）の保存タイミングについて』
'  │・記録ファイルは、アプリケーション終了時の状態を次回再開する時のために記録しておくファイルなので、単純に考えれば、アプリケーション終了時（すなわち《Form.Closing》イベント）に作成すれば良いということになる。
'  │・もちろん、本作を実行したまま、OSが終了する事態も考えられるけど、その時でもOSから各アプリケーションに終了要求が出されて《Form.Closing》イベントが発生するので問題ないと思っていた。
'  │・ところが、実際にOSを終了させてみると、なぜだかエラーが発生して正常終了できなくなる。
'  │→ そこで、別の経路（《Microsoft.Win32.SystemEvents.SessionEnding》イベント）でOSの終了を察知して、こちらでも処理できるようにしてみた。
'  │・この対策で、エラーは出なくなったけど、その原因は不明。イベントの発生タイミングに問題があるのか？ 例えば、《Form.Closing》イベント発生が遅すぎて、既にXML生成機能が終了してしまっているとか・・・。
'  │
'  │『《Form.TransparencyKey》の透過設定について』
'  │・透明にすると、デスクトップの画像を取り込む際などに、そこにあるコントロールも含めてフォーム全体が無視され下の様子を拡大できる。また、透けて見えている物を操作することも可能となる。← これは、WindowsXP環境での挙動（GPUに起因する問題？）のようで、Windows8.1では半透明フォームも取り込まれる。
'  │・しかし、実際に透過設定を行うと、いくつか問題が発生する。
'  │●問題点1・・・ビューアの《TransparencyKey》プロパティを設定した状態で、フォーム枠の有り無しを変更すると、デスクトップ画像が乱れる（タイトルバーの分ぐらいずれて復元される）現象が発生する。 
'  │└→ とりあえず、透過設定を一時的に解除してから枠を変更するようにすれば、回避できるみたい。
'  │●問題点2・・・MSDNの説明では、《TransparencyKey》プロパティに設定した色と同じ色のコントロールも透過すると書かれている。しかし、実際には、ほとんどの場合で透明にならない。コントロールが透明にならないのは逆に好都合なのだが、困ったことにごく稀に透明になることもあるみたい。再現性が低く、原因も対処法も不明。
'  │└→ この症状が出ると、拡大画像に含まれる透過色と同じ色の部分が抜けてしまう。（拡大画像をクリップボードにコピーしてペイントで見てみても透明になっていないので、この透過設定が原因だと考えられる。）
'  │・ビューアも、情報パネルの様に背景を透明にしたかったけど、このような不可解な動作をするため諦めた。
'  │
'  │『拡大画像と回避発動距離の最大サイズについて』
'  │・使用画面（作業領域の短辺）を基準として、拡大画像の最大サイズを35%以下とし、回避発動距離の最大サイズを15%以下とする。この両者を足して、50%以下でなけれならない。 → 50%超を指定すると、回避モードが“金魚”でビューアが画面外にはみ出してしまったり、どうしても見えない領域ができてしまったりする。
'  │・計算によって最大値を制限することで、解像度に合わせた設定値を用意する必要がなくなり、ユーザが誤ってビューアを大きくし過ぎる心配もなくなる。また、回避発動距離の最大値によって拡大画像の最大サイズを可変にしなかったのは、絶対的な基準を設けて、物事を単純化するため。
'  │※ただし、ビューアには、拡大画像のグリッドやタイトルバーなど様々な要素があるので、もしかすると少々はみ出してしまうことがあるかもしれない。
'  │
'  │『レジュームの表示位置について』
'  │・“手取り足取り”モードで前回表示していた位置を復元するために記憶する座標は、フォームの左上ではなく中央の座標とすることで、マルチスクリーンへの対応力を高めている。
'  │●理由1・・・これは、.NETでフォームが存在する画面のサイズを取得するとき、フォームの中心座標を基準に判断しているみたいだから。
'  │●理由2・・・ビューアのサイズによって、使用する画面が変わってしまう問題を回避するため。
'  │└→ 例えば、左にディスプレイ1(D1)、右にディスプレイ2(D2)と並んでいるとする。
'  │     小さいビューアをD1の右端から半分近くはみ出す位置に配置すると、使用画面は"D1"ということになる。このまま終了し、再び起動すると、起動直後はデザイン時の大きなフォームサイズとなっているので、中心座標がD2側にある状態になり、"D2"が使用画面として選択されてしまう。
'  │     このように、デザイン時と実際のサイズが異なることで、使用画面が違ってしまう問題があった。「ビューアのサイズを確定するには解像度が必要 → 解像度を得るにはビューアを作成して表示位置を確定しなければならない」という無限ループに陥ってしまう。
'  │     また、設定コンソールでビューアサイズを小さいものから大きいものへ変更したときにも、同じ問題が起きる。
'  │
'  │『回避や情報パネルの実装方法』
'  │・今作では、回避や情報パネルの実装に“継承”という機能を利用してみた。この方法が初めての自分には衝撃的だったけど、世間的には一般的な手段のようです。
'  │・モダンなオブジェクト指向の言語には、継承という概念があり、基本となるオブジェクトを拡張して簡単に使い回せるようになっている。そんなことは端から知っていて、.NETが標準で提供しているコントロールを拡張したりしていた。
'  │　ところが、継承の実力はそんなもんじゃなくて、シーケンス（処理の流れ）の切り替えもできてしまうことを知ってしまった。（知ったなら、やってみたい衝動に駆られる。）
'  │・この『継承によるシーケンスの切り替え』は、「大枠は同じだけど中身が少しずつ違う処理がいくつもある」場合に有用で、本作で言えば、回避や情報パネルにピッタリ。
'  │・各処理の共通点を“基底クラス”として用意 → それを継承しながら各々独自の部分を追記して“派生クラス”を作る → この派生クラスをとっかえひっかえインスタンスして呼び出す → 分岐命令を長々と書き連ねることなく異なる振る舞いをさせられる。
'  │！まさに、MSXのカートリッジを差し替える感覚（いや、ファミコンのカセットという方が通り易いかな？ いやいや、CDやDVDを入れ替えるの方が…）、いやもぉ～快感！！ 天晴、これ考えた人天才。これまた勉強になったわぁ。
'  │※ただし、実行ファイルは大きくなっちゃうようだ。
'  └────────────
'┌────┐
'│仕様要綱├──────────────────────
'└┬───┘
'  │『ダイレクトキーとコマンドパッドについて』
'  │・ダイレクトキーとコマンドパッドの2種類があるのは、ダイレクトでなくてもいい機能を一箇所に集めて、覚えないといけないキーの組み合わせを減らすため。
'  │・操作キー（ダイレクトキー含む）の定義は、機能を追加できるように、修飾子キーも含めて完全に独立させている。カーソルキーが、一つずつ定義されているのもそのため。
'  │
'  │『非表示時の拡大鏡の挙動について』
'  │・最新の状態で復帰するためや拡大画像やビューをコピーするために、裏で拡大画像・回避・情報パネル共に更新し続けている。それでも、タスク マネージャで見る限り、CPU使用率が軽減されるみたい。
'  │
'  │『ビュー（ビューア）サイズの変更方法の悩み』
'  │・現行方法では、絶対的なピクセル数にて指定している。
'  │　├→ 利点・・・実際の大きさを把握できる。解像度が違う画面へ移動しても（最大値を越さない限り）視野が変化しない。
'  │　└→ 欠点・・・使用する画面の解像度を常に監視しないといけない。解像度によっては、指定サイズと異なってしまう。
'  │・現行方法の欠点の改善策として、画面に対する相対的な大きさを割合で指定する方法を考えてみた。
'  │　├→ 利点・・・画面の解像度に対する制約を受けない。
'  │　└→ 欠点・・・異なる解像度の画面に移動するとサイズが変わってしまう。Single型となるため設定コンソールでは扱い難い。「nピクセル分大きくしたい」といった設定も困難。
'  │・相対的割合指定方式の欠点の改善策として、通常のフォームと同じく枠をドラッグしてサイズ変更する方法を考えてみた。
'  │　├→ 利点・・・直感的にサイズ変更できる。
'  │　└→ 欠点・・・切りのいいサイズに調整し難い。回避機能と相性が悪い。フォームの移動とリサイズの区別が難しい。
'  │・いずれの方法も欠点がそれなりに大きくて、スッキリ解決することができない。
'  └────────────





''' <summary>
''' メインシステム 兼 ビューア
''' </summary>
''' <remarks>ビューアの振る舞いをしながらシステム全体を統括する。基底クラスに拡張フォームの《UnProtrudeForm》を使用する。</remarks>
Public Class FormMain_Viewer

    ''' <summary>更新間隔の最小値</summary>
    Public Const REFRESH_INTERVAL_MINIMUM As Integer = 50

    ''' <summary>更新間隔の最大値</summary>
    Public Const REFRESH_INTERVAL_MAXIMUM As Integer = 1000


    ''' <summary>ビューの最大サイズ</summary>
    ''' <remarks>35%。[画面の短辺の50% - 回避発動距離の最大値]以下でなければいけない。（50%を超えると、回避モードが“金魚”でビューが画面外にはみ出してしまう。）</remarks>
    Public Const VIEW_SIZE_MAXIMAM As Single = 0.35F

    ''' <summary>ビュー寸法調整の最小値</summary>
    Public Const VIEW_ADJUST_MINIMUM As Integer = 32

    ''' <summary>ビュー寸法調整の最大値</summary>
    Public Const VIEW_ADJUST_MAXIMAM As Integer = 1024

    ''' <summary>倍率の最小値</summary>
    Public Const MAGNIFICATION_MINIMUM As Integer = 2

    ''' <summary>倍率の最大値</summary>
    Public Const MAGNIFICATION_MAXIMUM As Integer = 128


    ''' <summary>マウスカーソルとの接近許容距離の最小値</summary>
    Public Const NEAR_DISTANCE_MINIMUM As Integer = 1

    ''' <summary>マウスカーソルとの接近許容距離の最大値</summary>
    ''' <remarks>画面の縦横の短い方の半分からビューの最大サイズを引いた値</remarks>
    Public Const NEAR_DISTANCE_MAXIMUM As Integer = 50 - VIEW_SIZE_MAXIMAM * 100



    '===========================================================================================================



    ''' <summary>キャンバス</summary>
    Private Structure Canvas

        ''' <summary>ビットマップ</summary>
        Public Bmp As Bitmap

        ''' <summary>グラフィックス</summary>
        Public Grp As Graphics

    End Structure


    '-----------------------------------------------------------------------------------------------------------


    Private frmInfoPanel As FormInfoPanel_Base = Nothing                                       '情報パネルフォーム
    Private WithEvents frmCommandPad As FormCommandPad = New FormCommandPad With {.Owner = Me} 'コマンドパッドフォーム
    Private frmAbout As FormAbout = Nothing                                                    'バージョン情報フォーム
    Private frmSetup As FormSetupConsole = Nothing                                             '設定コンソールフォーム

    Private WithEvents AvoidPlanner As Avoid_Base = Nothing '逃避行請負人

    Private AreaSize As Size    '視野
    Private AreaCenter As Point '視野の中心点
    Private ViewOffset As Point 'ビュー描画用オフセット値

    Private TranscriptScreen As Canvas  '画面複写用キャンバス
    Private ImageScreen As Canvas       '拡大画像用キャンバス
    Private GridMarkerScreen As Canvas  '格子線&カーソルマーカー用キャンバス

    Private ImageMemory(,) As Integer '拡大画像記憶（拡大したピクセルの色を記録）

    Private UrgentReduction As Boolean = False 'ビューアの緊急縮小フラグ
    Private CaptureError As Boolean = False    'キャプチャ失敗フラグ



    '===========================================================================================================



    Private _UsingScreen As Rectangle
    Private _ViewSizeUpperLimit As Integer
    Private _ShootingLocation As Point = New Point(-1, -1)

    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ''' <summary>
    ''' 使用画面の作業領域
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property UsingScreen As Rectangle
        Get
            Return _UsingScreen
        End Get
    End Property

    ''' <summary>
    ''' 現在のビュー上限サイズ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ViewSizeUpperLimit As Integer
        Get
            Return _ViewSizeUpperLimit
        End Get
    End Property

    ''' <summary>
    ''' 現在の注目点
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>撮影場所（ビューに表示中のマウスカーソル）の座標。更新間隔によってずれが生じるので、注目点と呼ぶ。</remarks>
    Public ReadOnly Property ShootingLocation As Point
        Get
            Return _ShootingLocation
        End Get
    End Property



    '===========================================================================================================



    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '=== システムイベント関連付け ===
        AddHandler Microsoft.Win32.SystemEvents.SessionEnding, AddressOf SystemEvents_SessionEnding                   'Windowsの終了 ┬イベントを捕捉する。
        AddHandler Microsoft.Win32.SystemEvents.DisplaySettingsChanged, AddressOf SystemEvents_DisplaySettingsChanged '画面設定の変更┘

        '=== 初期設定 ===
        '--- 設定ファイル読み込み ---
        Try
            Dim message As String = SettingLoad(SettingData)
            If message <> "" Then MessageBox.Show("下記の設定ファイル項目が異常だったため、修正して起動します。" & vbCrLf & vbCrLf & message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch ex As Exception
            MessageBox.Show("設定ファイルの読み込み・復元中に例外が発生しました。" & vbCrLf & "既定値を使用して起動します。" & vbCrLf & vbCrLf & ex.Message,
                            Me.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try

        '--- 記録ファイル読み込み ---
        Try
            ResumeLoad(ResumeData)
        Catch ex As Exception
            MessageBox.Show("記録ファイルの読み込み・復元中に例外が発生しました。" & vbCrLf & "既定値を使用して起動します。" & vbCrLf & vbCrLf & ex.Message,
                            Me.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try
        ResumeData.Memo() '終了時に変更を判断するため、起動時の値を記憶しておく。

        '--- タスクトレイにアイコンを登録 ---
        ntfTask.Icon = My.Resources.LupeOn
        ntfTask.Visible = True

    End Sub

    Private Sub FormMain_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown

        '=== 拡大鏡を作成 ===
        AllNew()

        '=== 各種タイマ始動 ===
        tmrRefresh.Enabled = True
        tmrOpeKeyScan.Enabled = True

    End Sub

    Private Sub FormMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        '=== 通常終了時の記録保存 ===
        ResumeSaveing()

    End Sub

    Private Sub FormMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed

        '=== システムイベント関連付け解除 ===
        RemoveHandler Microsoft.Win32.SystemEvents.SessionEnding, AddressOf SystemEvents_SessionEnding
        RemoveHandler Microsoft.Win32.SystemEvents.DisplaySettingsChanged, AddressOf SystemEvents_DisplaySettingsChanged

        '=== タスクトレイからアイコン消去 ===
        ntfTask.Visible = False

    End Sub

    Private Sub FormMain_Activated(sender As Object, e As System.EventArgs) Handles Me.Activated

        '=== 設定中ならフォーカスを設定コンソールに渡す ===
        '（情報パネルの生成でこっちにフォーカスが移ってくるので）
        If frmSetup IsNot Nothing AndAlso frmSetup.Visible Then
            frmSetup.Select()
        End If

    End Sub

    Private Sub FormMain_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        'この[ResizeEnd]イベントは、ユーザがフォームを移動し終えた時にも発生する（コードでプロパティを変更しても発生しない）ので、“手動モード時の移動完了”イベントとして利用している。

        '=== 〔手取り足取り〕（手動）モード時の処理 ===
        If SettingData.Lupe.Avoid.Type = AvoidActionTypes.Passive Then

            ResumeData.ViewerLocate = Point.Add(Me.Location, New Size(Me.Width \ 2, Me.Height \ 2)) 'ビューアの表示位置（フォームの中心座標）を記録する。

            If SetBasisParameter() Then MakeViewer() '使用する画面の作業領域サイズの変化を調べて、必要ならビューアを再作成する。
            UnProtrudeRepositioner(UsingScreen)

        End If

    End Sub

    Private Sub SystemEvents_SessionEnding(ByVal sender As Object, ByVal e As Microsoft.Win32.SessionEndingEventArgs)

        '=== 常駐しながらWindowsが終了した場合の記録保存 ===
        ResumeSaveing()

    End Sub

    Private Sub SystemEvents_DisplaySettingsChanged(sender As Object, e As System.EventArgs)

        '=== 画面設定が変更された ===
        If SetBasisParameter() Then MakeViewer() '使用する画面の作業領域サイズの変化を調べて、必要ならビューアを再作成する。
        UnProtrudeRepositioner(UsingScreen)

    End Sub

    Private Sub ntfTask_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ntfTask.MouseClick

        '=== 表示/非表示の切り替え ===
        If e.Button = Windows.Forms.MouseButtons.Left Then
            tsmHide.PerformClick()
        End If

    End Sub

    Private Sub cmsTaskMenu_Opening(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles cmsTaskMenu.Opening

        '=== 子フォームの表示中は、コンテクストメニューを開けない ===
        Dim openSubForm As Form = Nothing
        If frmAbout IsNot Nothing Then
            openSubForm = frmAbout
        ElseIf frmSetup IsNot Nothing Then
            openSubForm = frmSetup
        End If

        If openSubForm IsNot Nothing Then
            e.Cancel = True
            ntfTask.ShowBalloonTip(5000, "『" & openSubForm.Text & "』を使用中です。", "閉じてから、他の作業へ移ってください。", ToolTipIcon.Info)
            openSubForm.Activate()
        End If

    End Sub

    Private Sub tsmQuit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmQuit.Click

        '=== 終了する ===
        Me.Close()

    End Sub

    Private Sub tsmAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmAbout.Click

        '=== バージョン情報を表示する ===
        frmAbout = New FormAbout
        frmAbout.ShowDialog(Me) '機能的には“モードレス”で事足りるけど、あえて“モーダル”にして閉じられるのを待つ。また、拡大鏡に隠れてはいけないので、ビューアを《Owner》にする。
        frmAbout.Dispose()
        frmAbout = Nothing

    End Sub

    Private Sub tsmSetting_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmSetting.Click

        '=== 設定 ===
        Me.TopMost = False '最上位の状態で設定コンソールを表示すると、（操作キー一覧等の）ToolTipがフォームの裏側に表示されてしまい読めない不具合が発生するため、一時的に解除する。

        frmSetup = New FormSetupConsole
        If frmSetup.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            AllNew()
        End If
        frmSetup.Dispose()
        frmSetup = Nothing

        Me.TopMost = True

    End Sub

    Private Sub tsmHide_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmHide.CheckedChanged

        '=== 表示/非表示の切り替え ===
        Me.Visible = Not tsmHide.Checked
        ntfTask.Icon = If(Me.Visible, My.Resources.LupeOn, My.Resources.LupeOff) 'タスクトレイのアイコンを変更

    End Sub

    Private Sub tmrRefresh_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrRefresh.Tick

#If DEBUG Then

        '=== 拡大鏡更新（デバッグ用） ===
        Static total As Integer = 0, tank As New Queue '平均更新作業時間計測用

        If SettingData.Lupe.Avoid.Type = AvoidActionTypes.Passive Then '手動時には、更新に掛かった時間を計測する。

            Dim cycle As Integer = 1000 \ tmrRefresh.Interval '1秒間に更新される予定回数
            Dim sw As New Stopwatch 'ストップウォッチ
            sw.Start()
            RefreshLupe()
            sw.Stop()

            If tank.Count = cycle Then total -= tank.Dequeue
            total += sw.ElapsedMilliseconds
            tank.Enqueue(sw.ElapsedMilliseconds)

            Me.Text = String.Format("{0:D4}({1:D4})ms", total \ tank.Count, sw.ElapsedMilliseconds) '直前1秒間の平均と今回の更新作業時間を表示する。

        Else

            RefreshLupe()

        End If

#Else

        '=== 拡大鏡更新（リリース用） ===
        RefreshLupe()

#End If

    End Sub

    Private Sub picView_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles picView.Paint

        With e.Graphics

            '=== ビュー描画 ===
            .DrawImage(ImageScreen.Bmp, ViewOffset)
            .DrawImage(GridMarkerScreen.Bmp, ViewOffset) 'グリッドやマーカーがない時、透明な画像を合成することになるけど、この処理は十分高速であり、毎回条件判定するより処理が安定している（むしろ条件分岐がない分早いような）気がする。気のせいかもしれんけど…。

            ''サイト(Sight)描画 XOR版 ※劇重！！
            'Dim r As Integer = SettingData.Lupe.Viewer.Magnification * Math.Min(AreaCenter.X, AreaCenter.Y)
            'Dim b As Point = picView.PointToScreen(New Point(AreaCenter.X * SettingData.Lupe.Viewer.Magnification, AreaCenter.Y * SettingData.Lupe.Viewer.Magnification))
            'ControlPaint.DrawReversibleLine(b, b + New Point(-r, -r), Color.White)
            'b.X += SettingData.Lupe.Viewer.Magnification
            'ControlPaint.DrawReversibleLine(b, b + New Point(r, -r), Color.White)
            'b.Y += SettingData.Lupe.Viewer.Magnification
            'ControlPaint.DrawReversibleLine(b, b + New Point(r, r), Color.White)
            'b.X -= SettingData.Lupe.Viewer.Magnification
            'ControlPaint.DrawReversibleLine(b, b + New Point(-r, r), Color.White)

            '=== 注意喚起枠の描画 ===
            Dim i As Integer 'ループ
            Dim alpha As Integer, pen0, pen1 As Pen, rect As Rectangle '不透明度とペンと矩形

            '--- 設定の試用中・解像度変更に伴うビューアの緊急縮小中 ---
            If CaptureError Then
                alpha = &HFF
            ElseIf frmSetup IsNot Nothing AndAlso frmSetup.CheckTrying Then
                alpha = &HAA
            ElseIf UrgentReduction Then
                alpha = &H44
            End If

            If alpha > 0 Then

                '工事現場の黄黒帯風縁取り
                pen0 = New Pen(New SolidBrush(Color.FromArgb(alpha, &HFF, &HEE, 0))) With {.DashStyle = Drawing2D.DashStyle.Custom, .DashPattern = New Single() {8.0F, 16.0F}}
                pen1 = pen0.Clone
                pen1.Color = Color.FromArgb(alpha, Color.Black)
                pen1.DashOffset = 12.0F

                With picView.ClientRectangle
                    rect = New Rectangle(.X + 3, .Top, .Width - 4, .Height - 4)
                End With

                For i = 0 To 3
                    .DrawRectangle(pen0, rect)
                    .DrawRectangle(pen1, rect)
                    rect.Offset(-1, 1)
                    '《.DashOffset》プロパティを使っても斜めストライプを描画できるけど、四角形《rect》を1ドットずつずらした方が簡単。その代わり、角が描けない…たいした問題じゃないよね。
                Next

            End If

            '--- コマンドパッド使用中による一時停止 ---
            If frmCommandPad.Visible Then

                '縁をフェードアウト
                pen0 = New Pen(Brushes.Black)

                rect = picView.ClientRectangle
                rect.Width -= 1 : rect.Height -= 1 '実サイズよりも縦横ともに1ドット大きいので引いておく。

                For i = 0 To 7
                    pen0.Color = Color.FromArgb(&HFF - &H22 * i, pen0.Color)
                    .DrawRectangle(pen0, rect)
                    rect.Inflate(-1, -1)
                Next

                'Cursors.WaitCursor.Draw(e.Graphics, New Rectangle(2, 2, picView.ClientSize.Width, picView.ClientSize.Height)) 'WindowsXPのころは“砂時計”だったけど、今ではただの輪っかになっちゃって意味が分かりにくくなったなぁ。
                'Cursors.No.Draw(e.Graphics, New Rectangle(2, 2, picView.ClientSize.Width, picView.ClientSize.Height)) '“禁止”ではないので、これもちょっと違う気がする。カーソルの流用は、断念するしかないか。

            End If

        End With

    End Sub

    ''' <summary>
    ''' 拡大鏡を刷新する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub AllNew()

        '=== 隠匿解除 ===
        tsmHide.Checked = False

        '=== ビューア作成 ===
        SetBasisParameter()
        MakeViewer()
        ImageCapture()
        Enlarge()

        '=== 逃避行プランナー交代 ===
        If AvoidPlanner IsNot Nothing Then AvoidPlanner.Dispose()

        Select Case SettingData.Lupe.Avoid.Type
            Case AvoidActionTypes.Passive
                AvoidPlanner = Nothing
                UnProtrudeRepositioner(New Point(ResumeData.ViewerLocate.X - Me.Width \ 2, ResumeData.ViewerLocate.Y - Me.Height \ 2), UsingScreen) '前回終了時の位置へ
            Case AvoidActionTypes.ScreenSide
                AvoidPlanner = New Avoid_ScreenSide(Me)
            Case AvoidActionTypes.RightTriangle
                AvoidPlanner = New Avoid_RightTriangle(Me)
            Case AvoidActionTypes.ScreenCorner
                AvoidPlanner = New Avoid_ScreenCorner(Me)
            Case AvoidActionTypes.Randezvous
                AvoidPlanner = New Avoid_Randezvous(Me)
        End Select

        '=== 情報パネル交換 ===
        If frmInfoPanel IsNot Nothing Then frmInfoPanel.Close()

        Select Case SettingData.Lupe.InformationPanel.Style
            Case InfoPanelStyles.Full
                frmInfoPanel = New FormInfoPanel_Full
            Case InfoPanelStyles.Bar
                frmInfoPanel = New FormInfoPanel_Bar
            Case InfoPanelStyles.Square
                frmInfoPanel = New FormInfoPanel_Square
            Case InfoPanelStyles.Twig
                frmInfoPanel = New FormInfoPanel_Twig
            Case InfoPanelStyles.Mini
                frmInfoPanel = New FormInfoPanel_Mini
            Case Else
                frmInfoPanel = Nothing
        End Select

        If frmInfoPanel IsNot Nothing Then frmInfoPanel.Show(Me)

    End Sub

    ''' <summary>
    ''' 拡大鏡に必要な基礎パラメータを決める
    ''' </summary>
    ''' <returns>解像度が変化していたら[True]を返す。</returns>
    ''' <remarks></remarks>
    Private Function SetBasisParameter() As Boolean

        Dim changed As Boolean = False '変更フラグ
        Dim tempScreen As Rectangle '拡大鏡を使用する画面の作業領域

        '=== 現在の作業領域を取得 ===
        tempScreen = Screen.GetWorkingArea(If(SettingData.Lupe.Avoid.Cross AndAlso SettingData.Lupe.Avoid.Type <> AvoidActionTypes.Passive, MousePosition, ResumeData.ViewerLocate))

        '=== 対象作業領域範囲の変化を確認 ===
        If UsingScreen <> tempScreen Then

            changed = True

            _UsingScreen = tempScreen

            '--- 基礎パラメータ算出 ---
            With UsingScreen
                _ViewSizeUpperLimit = If(.Height < .Width,
                                         .Height * VIEW_SIZE_MAXIMAM - SystemInformation.Border3DSize.Height * 2,
                                         .Width * VIEW_SIZE_MAXIMAM - SystemInformation.Border3DSize.Width * 2) '最大拡大画像サイズを算出する。画面の短辺を基準とする。
                '※厳密には、右下端のグリッド線の分も考慮するべきだけど、まだグリッド線の有無が確定していないのと、しょせん1ドットなので無視する。 
                '※最大の状態でも画面の1/2を占有することはないので、“手取り足取り”モードでのタイトルバー&フレーム枠&余白を考慮する必要はない。
                '《SystemInformation.Border3DSize.Height * 2》,《SystemInformation.Border3DSize.Width * 2》は、PictureBoxの上下または左右の枠幅。
            End With

            '--- 設定コンソールへの対応 ---
            If frmSetup IsNot Nothing Then frmSetup.LimitChecker()

        End If

        Return changed

    End Function

    ''' <summary>
    ''' 設定内容に合わせてビューアを作る
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MakeViewer()

        Dim gridSpace As Integer = If(SettingData.Lupe.Viewer.Grid.Enabled, 1, 0) '右端と下端の格子線を表示するスペースの有無。

        With SettingData.Lupe

            '=== フォーム形状 ===
            Me.FormBorderStyle = If(.Avoid.Type = AvoidActionTypes.Passive, Windows.Forms.FormBorderStyle.FixedToolWindow, Windows.Forms.FormBorderStyle.None)
            Me.Size += New Size(Math.Min(.Viewer.ViewSize.Width, ViewSizeUpperLimit) + gridSpace, Math.Min(.Viewer.ViewSize.Height, ViewSizeUpperLimit) + gridSpace) - picView.ClientSize

            '=== 拡大パラメータ算出 ===
            With .Viewer
                AreaSize = New Size(Math.Ceiling((picView.ClientSize.Width - 1) / .Magnification),
                                    Math.Ceiling((picView.ClientSize.Height - 1) / .Magnification)) '視野
                'AreaSize -= New Size((AreaSize.Width Mod 2 = 0), (AreaSize.Height Mod 2 = 0)) 'マウスマーカーが中心に来るように奇数化。システム的には偶数でもエラーにはならない。
                AreaCenter = New Point(AreaSize.Width \ 2, AreaSize.Height \ 2) '視野の中心

                '--- 設定内容から調整されたか？ ---
                UrgentReduction = (picView.ClientSize.Width <> .ViewSize.Width + gridSpace OrElse picView.ClientSize.Height <> .ViewSize.Height + gridSpace)
            End With

            '=== その他設定 ===
            tmrRefresh.Interval = .RefreshInterval '更新間隔

            ntfTask.Text = String.Format("『{1}』{0}寸法:{2}*{3}({4}*{5}){0}視野:{6}*{7}{0}倍率:×{8}",
                                         vbCrLf,
                                         My.Application.Info.Title,
                                         picView.ClientSize.Width - gridSpace, picView.ClientSize.Height - gridSpace,
                                         .Viewer.ViewSize.Width, .Viewer.ViewSize.Height,
                                         AreaSize.Width, AreaSize.Height,
                                         .Viewer.Magnification) 'パラメータ表示

        End With

        '=== 下地作成 ===
        With picView
            .BackColor = SettingData.Lupe.Viewer.Foundation.MainColor
            If .Image IsNot Nothing Then
                .Image.Dispose()
                .Image = Nothing
            End If

            '--- 模様を描く ---
            If SettingData.Lupe.Viewer.Foundation.Enabled Then
                .Image = New Bitmap(.ClientSize.Width, .ClientSize.Height)
                '《Image》プロパティに背景画像を直接置くと非常に便利！！
                '  ├ メリットその1. 再描画など何もしなくても、表示され続ける。《Nothing》を入れれば消える。
                '  └ メリットその2. 《Refresh》メソッドを呼べば、《Paint》イベントの《e.Graphics》を用いて描いた分だけが消える。

                Using g As Graphics = Graphics.FromImage(.Image)
                    g.FillRectangle(
                        New Drawing2D.HatchBrush(SettingData.Lupe.Viewer.Pattern, SettingData.Lupe.Viewer.Foundation.SubColor, Color.Transparent),
                        .ClientRectangle
                    ) '背景色を指定しないと黒になってしまうのは不便。なぜ、透明にしなかったんだ！
                End Using
            End If
        End With

        '=== 各キャンバス作成 ===
        '--- デスクトップ画像取り込み用 ---
        With TranscriptScreen
            If .Grp IsNot Nothing Then .Grp.Dispose()
            If .Bmp IsNot Nothing Then .Bmp.Dispose()

            .Bmp = New Bitmap(AreaSize.Width, AreaSize.Height)
            .Grp = Graphics.FromImage(.Bmp)
        End With

        '--- 拡大画像用 ---
        With ImageScreen
            If .Grp IsNot Nothing Then .Grp.Dispose()
            If .Bmp IsNot Nothing Then .Bmp.Dispose()

            .Bmp = New Bitmap(AreaSize.Width * SettingData.Lupe.Viewer.Magnification, AreaSize.Height * SettingData.Lupe.Viewer.Magnification)
            .Grp = Graphics.FromImage(.Bmp)
            .Grp.CompositingMode = Drawing2D.CompositingMode.SourceCopy '透明色を合成しない（透明色で描き換えられる）ようにする。
        End With

        '--- 格子線&カーソルマーカー用 ---
        With GridMarkerScreen
            If .Grp IsNot Nothing Then .Grp.Dispose()
            If .Bmp IsNot Nothing Then .Bmp.Dispose()

            .Bmp = New Bitmap(ImageScreen.Bmp.Width + gridSpace, ImageScreen.Bmp.Height + gridSpace)
            .Grp = Graphics.FromImage(.Bmp)

            Dim p As Pen, rect As Rectangle '絵具

            '格子描画
            If SettingData.Lupe.Viewer.Grid.Enabled Then
                p = New Pen(SettingData.Lupe.Viewer.Grid.MainColor)
                For i As Integer = 0 To .Bmp.Width Step SettingData.Lupe.Viewer.Magnification
                    .Grp.DrawLine(p, i, 0, i, .Bmp.Height)
                Next
                For i As Integer = 0 To .Bmp.Height Step SettingData.Lupe.Viewer.Magnification
                    .Grp.DrawLine(p, 0, i, .Bmp.Width, i)
                Next
            End If

            'サイト描画
            '目立ち過ぎないように無影とした。
            If SettingData.Lupe.Viewer.Sight.Enabled Then
                Dim x, y As Integer '座標計算用

                '縦横
                rect = Rectangle.Truncate(.Bmp.GetBounds(Drawing.GraphicsUnit.Pixel))
                p = New Pen(SettingData.Lupe.Viewer.Sight.MainColor)
                For i As Integer = 1 To 4
                    x = AreaCenter.X * SettingData.Lupe.Viewer.Magnification + Choose(i, 0, SettingData.Lupe.Viewer.Magnification, 0, 0)
                    y = AreaCenter.Y * SettingData.Lupe.Viewer.Magnification + Choose(i, 0, 0, SettingData.Lupe.Viewer.Magnification, 0)
                    Do
                        .Grp.DrawLine(p,
                                      x,
                                      y,
                                      x + SettingData.Lupe.Viewer.Magnification * Choose(i, 1, 0, 1, 0),
                                      y + SettingData.Lupe.Viewer.Magnification * Choose(i, 0, 1, 0, 1))
                        x += SettingData.Lupe.Viewer.Magnification * Choose(i, 0, +1, 0, -1)
                        y += SettingData.Lupe.Viewer.Magnification * Choose(i, -1, 0, +1, 0)
                    Loop While rect.Contains(x, y)
                Next

                '斜め
                If SettingData.Lupe.Viewer.SightX Then
                    For i As Integer = 1 To 4
                        x = AreaCenter.X * SettingData.Lupe.Viewer.Magnification + Choose(i, 0, SettingData.Lupe.Viewer.Magnification, SettingData.Lupe.Viewer.Magnification, 0)
                        y = AreaCenter.Y * SettingData.Lupe.Viewer.Magnification + Choose(i, 0, 0, SettingData.Lupe.Viewer.Magnification, SettingData.Lupe.Viewer.Magnification)
                        Do
                            .Grp.DrawLine(p, x - 2, y, x + 2, y) '-
                            .Grp.DrawLine(p, x, y - 2, x, y + 2) '|
                            x -= SettingData.Lupe.Viewer.Magnification * Choose(i, -1, +1, +1, -1)
                            y -= SettingData.Lupe.Viewer.Magnification * Choose(i, -1, -1, +1, +1)
                        Loop While rect.Contains(x, y)
                    Next
                End If
            End If

            'マーク描画
            If SettingData.Lupe.Viewer.Mark.Enabled Then
                rect = New Rectangle(AreaCenter.X * SettingData.Lupe.Viewer.Magnification, AreaCenter.Y * SettingData.Lupe.Viewer.Magnification, SettingData.Lupe.Viewer.Magnification, SettingData.Lupe.Viewer.Magnification)
                p = New Pen(SettingData.Lupe.Viewer.Mark.MainColor)
                .Grp.DrawRectangle(p, rect)
                rect.Inflate(1, 1)
                p.Color = ColorAdjuster(SettingData.Lupe.Viewer.Mark.MainColor, SettingData.Lupe.Viewer.Mark.SubColor)
                .Grp.DrawRectangle(p, rect)
            End If

            'オフセット値算出
            ViewOffset = New Point((picView.ClientSize.Width - .Bmp.Width) \ 2, (picView.ClientSize.Height - .Bmp.Height) \ 2) 'グリッドスペースを考慮する必要があるので、ここのBMP寸法から計算する。
        End With

        '=== 仮想拡大画像の初期化 ===
        ReDim ImageMemory(AreaSize.Width, AreaSize.Height) 'MSDNに「Integer型の既定値は0」と書かれているので、わざわざゼロクリアする必要はない？
        'For y As Integer = 0 To VirtualView.GetUpperBound(1)
        '    For x As Integer = 0 To VirtualView.GetUpperBound(0)
        '        VirtualView(x, y) = 0
        '    Next
        'Next

    End Sub

    ''' <summary>
    ''' 色を調整する
    ''' </summary>
    ''' <param name="Base">基本色</param>
    ''' <param name="Adjust">調整値</param>
    ''' <returns>調整された色</returns>
    ''' <remarks></remarks>
    Private Function ColorAdjuster(Base As Color, Adjust As Color) As Color

        Return Color.FromArgb(CInt(Base.A) * Adjust.A \ &HFF,
                              CInt(Base.R) * Adjust.R \ &HFF,
                              CInt(Base.G) * Adjust.G \ &HFF,
                              CInt(Base.B) * Adjust.B \ &HFF)

    End Function

    ''' <summary>
    ''' 拡大鏡を更新する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RefreshLupe()

        '=== ビューアを更新 ===
        If Not frmCommandPad.Visible Then
            _ShootingLocation = System.Windows.Forms.Cursor.Position 'マウスカーソル位置を取得する。
            ImageCapture() 'コマンドパッド使用中はキャプチャしない。コマンドパッドが映り込んでしまうため。
        End If
        Enlarge()

        '=== 情報パネルの表示内容を更新 ===
        If frmInfoPanel IsNot Nothing Then frmInfoPanel.PrintParameter(ShootingLocation, GetLocationColor)

        '=== 例外停止 ===
        If CaptureError Then
            tmrRefresh.Stop()
            MessageBox.Show("画面を取り込めない状態が発生したため、更新を停止しました。" & vbCrLf & "[OK]をクリックすると、再開します。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Information)
            CaptureError = False
            tmrRefresh.Start()
        End If

    End Sub

    ''' <summary>
    ''' 対象領域を撮影する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ImageCapture()

        With TranscriptScreen.Grp

            .Clear(Color.Transparent) '←《CompositingMode》プロパティを指定しても、《CopyFromScreen》の結果は合成されてしまうので、透明色でクリアしておく必要がある。

            Try
                .CopyFromScreen(ShootingLocation - AreaCenter, New Point(0, 0), AreaSize) '←この1行だけでスクリーンショットできちゃう！！
            Catch
                CaptureError = True 'スクリーンセーバーが走るとキャプチャに失敗するみたい。
            End Try

        End With

    End Sub

    ''' <summary>
    ''' 拡大画像を作成する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Enlarge()

        Static pixelBrush As New SolidBrush(Color.Black) 'ピクセル描画用ブラシ（静的変数とすることで、毎回インスタンシングしない分速くならないかなぁと思って。色はループ内で書き換えられるので、引数にはダミーを渡す。）
        Dim scale As Integer = SettingData.Lupe.Viewer.Magnification '1ピクセルの大きさ（何回もプロパティにアクセスしなくていいように変数を用意してみた。）

        For y As Integer = 0 To AreaSize.Height - 1
            For x As Integer = 0 To AreaSize.Width - 1

                pixelBrush.Color = TranscriptScreen.Bmp.GetPixel(x, y) '色を取得する。

                If ImageMemoryChecker(x, y, pixelBrush.Color.ToArgb) Then _
                    ImageScreen.Grp.FillRectangle(pixelBrush, x * scale, y * scale, scale, scale) '前回の更新時と違っていたら、ピクセルを描画する。（なぜかわからないけど、判定と更新をプロシージャ化したら速くなった気がする。）
                '自作ベンチマークで測ってみたら、《End If》無しの方が、若干速いみたい。

            Next
        Next

        picView.Refresh() '各種オーバーレイ表示の喚起も兼ねる。《Invalidate》でも構わないみたい。

    End Sub

    ''' <summary>
    ''' 拡大画像記憶との変化判定する
    ''' </summary>
    ''' <param name="X">調べたいピクセルのX座標</param>
    ''' <param name="Y">調べたいピクセルのY座標</param>
    ''' <param name="ARGB">比較する色</param>
    ''' <returns>変化があれば《True》、なければ《False》</returns>
    ''' <remarks>違う色ならば更新します。</remarks>
    Private Function ImageMemoryChecker(X As Integer, Y As Integer, ARGB As Integer) As Boolean
        '引数《X》,《Y》の代りに、《ImageMemory(X, Y)》の中身を参照渡しした方が速いかと思ったけど、そうでもなかった。

        If ImageMemory(X, Y) <> ARGB Then
            ImageMemory(X, Y) = ARGB '記録を更新する。
            Return True
        Else
            Return False
        End If

    End Function

    ''' <summary>
    ''' 注目点の色を取得する
    ''' </summary>
    ''' <returns>色コード(ARGB値)</returns>
    ''' <remarks></remarks>
    Private Function GetLocationColor() As Integer

        Return ImageMemory(AreaCenter.X, AreaCenter.Y)

    End Function

    ''' <summary>
    ''' 記録項目を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ResumeSaveing()

        '=== 起動時に対して変更があれば保存 ===
        If ResumeData.Modified Then

            Try
                ResumeSave()
            Catch ex As Exception
                MessageBox.Show("記録ファイルの保存中に例外が発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try

            ResumeData.Memo()

        End If

    End Sub

End Class