﻿''' <summary>
''' 情報パネル ～ 基底クラス
''' </summary>
''' <remarks>派生専用。</remarks>
Public MustInherit Class FormInfoPanel_Base

    ''' <summary>フォントサイズの最小値</summary>
    Public Const FONT_SIZE_MINIMUM As Single = 6.0F

    ''' <summary>フォントサイズの最大値</summary>
    Public Const FONT_SIZE_MAXIMUM As Single = 20.0F



    '===========================================================================================================



    ''' <summary>
    ''' 用意された情報パネル形態のコレクション
    ''' </summary>
    ''' <remarks>設定コンソールの[ComboBox]用</remarks>
    Public Shared ReadOnly PreparedLooks As List(Of PatternItem(Of InfoPanelStyles)) = {New PatternItem(Of InfoPanelStyles) With {.Caption = "表示しない", .TypeID = InfoPanelStyles.Unvisible},
                                                                                        New PatternItem(Of InfoPanelStyles) With {.Caption = "みたらし団子", .TypeID = InfoPanelStyles.Full},
                                                                                        New PatternItem(Of InfoPanelStyles) With {.Caption = "若あゆ", .TypeID = InfoPanelStyles.Bar},
                                                                                        New PatternItem(Of InfoPanelStyles) With {.Caption = "きんつば", .TypeID = InfoPanelStyles.Square},
                                                                                        New PatternItem(Of InfoPanelStyles) With {.Caption = "八つ橋", .TypeID = InfoPanelStyles.Twig},
                                                                                        New PatternItem(Of InfoPanelStyles) With {.Caption = "五色豆", .TypeID = InfoPanelStyles.Mini}}.ToList



    '===========================================================================================================



    ''' <summary>
    ''' ラベルを配置する
    ''' </summary>
    ''' <remarks>派生クラス側で実装。</remarks>
    Protected MustOverride Sub LabelArrange()

    ''' <summary>
    ''' 情報を掲示する
    ''' </summary>
    ''' <param name="Locate">注目点の座標</param>
    ''' <param name="ColorCode">注目点の色</param>
    ''' <remarks>派生クラス側で実装。</remarks>
    Public MustOverride Sub PrintParameter(Locate As Point, ColorCode As Integer)
    '注目点の座標と色を取りに行くコードを、全ての派生クラスで書くのは面倒くさいので、呼び出し時に渡してもらうことにした。呼び出し元も1か所だけだし。
    'オブジェクト指向的には、これが正統派なのかな？ でも、呼び出し元が意識しなくても必要な要素が自動的に揃えられて結果が得られる方が、個人的には好みなんだけど。違う値を渡してしまう間違えもしなくて済むし。
    'まぁ、一人で全て作っている（管理している）から言えるんでしょうね。

    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    Private Sub FormInfoPanel_Base_Load(sender As Object, e As System.EventArgs) Handles Me.Load

        '=== ラベル幅の確定 ===
        PrintParameter(New Point(0, 0), 0) 'ダミーの文字列を表示することで、《Label.Width》プロパティの値を確定させる。

        '=== フォント設定 ===
        'For Each item As Label In Me.Controls
        '    item.Font = SettingData.Lupe.InformationPanel.Font.GetFont
        'Next
        Me.Font = SettingData.Lupe.InformationPanel.Font.GetFont 'フォームのフォントを設定することで、一括変更できるみたい。

        '=== ラベル配置 ===
        LabelArrange()

        '=== 移動始動 ===
        Me.AutomaticFloat() '初期位置を確定
        tmrAct.Start()

    End Sub

    Private Sub FormInfoPanel_Base_Activated(sender As Object, e As System.EventArgs) Handles Me.Activated

        '=== [Alt]+[F4]対策 ===
        '[Alt]+[F4]が押されて情報パネルが消えてしまうのを回避するために常に親にフォーカスを返しておく。
        '設定コンソールのような、フォーカスを持たない方式では上手く行かなかった。
        Owner.Select()

    End Sub

    Private Sub tmrAct_Tick(sender As System.Object, e As System.EventArgs) Handles tmrAct.Tick

        '=== ビューアの周りをうまく立ち回る ===
        AutomaticFloat()

        '=== 状態をビューアに合わせる ===
        If Me.Visible <> Owner.Visible Then Me.Visible = Owner.Visible
        If Me.TopMost <> Owner.TopMost Then Me.TopMost = Owner.TopMost
        If Me.Opacity <> Owner.Opacity Then Me.Opacity = Owner.Opacity

    End Sub


    '-----------------------------------------------------------------------------------------------------------


    ''' <summary>
    ''' ラベルの配置
    ''' </summary>
    ''' <param name="MoveLabel">動かすラベル</param>
    ''' <param name="Basis">基準とするラベル（《Nothing》を指定すると(0,0)に配置される)</param>
    ''' <param name="Align">基準ラベルに対する配置方向</param>
    ''' <param name="AdjustX">水平方向の調整値</param>
    ''' <param name="AdjustY">垂直方向の調整値</param>
    ''' <remarks></remarks>
    Protected Sub LabelDisposition(MoveLabel As Label, Optional Basis As Label = Nothing, Optional Align As ContentAlignment = ContentAlignment.MiddleCenter, Optional AdjustX As Integer = 0, Optional AdjustY As Integer = 0)

        With MoveLabel

            If Basis Is Nothing Then

                '=== 左上隅 ===
                .Location = Point.Empty

            Else

                '=== 基準ラベルに対して上下左右 ===
                Select Case Align
                    Case ContentAlignment.TopLeft, ContentAlignment.MiddleLeft, ContentAlignment.BottomLeft
                        .Left = Basis.Left - .Width
                    Case ContentAlignment.TopCenter, ContentAlignment.MiddleCenter, ContentAlignment.BottomCenter
                        .Left = Basis.Left
                    Case ContentAlignment.TopRight, ContentAlignment.MiddleRight, ContentAlignment.BottomRight
                        .Left = Basis.Right
                End Select

                Select Case Align
                    Case ContentAlignment.TopLeft, ContentAlignment.TopCenter, ContentAlignment.TopRight
                        .Top = Basis.Top - .Height
                    Case ContentAlignment.MiddleLeft, ContentAlignment.MiddleCenter, ContentAlignment.MiddleRight
                        .Top = Basis.Top
                    Case ContentAlignment.BottomLeft, ContentAlignment.BottomCenter, ContentAlignment.BottomRight
                        .Top = Basis.Bottom
                End Select

            End If

            '=== 調整 ===
            .Left += AdjustX
            .Top += AdjustY

        End With

    End Sub

    ''' <summary>
    ''' オーナー（ビューア）の周囲を回り込む
    ''' </summary>
    ''' <remarks>回避モードにとらわれず、ビューアとマウスカーソルの位置関係や作業領域内でのビューアの位置から、最適な場所へ移動する。</remarks>
    Private Sub AutomaticFloat()

        'M0   |   M1    ●M0 ～ M3 : ビューアの中心から見たマウスカーソルの位置。
        '   |0 1|       ● 0 ～  7 : 情報パネルを表示する位置。左右反転（奇数-1, 偶数+1）、上下反転（3以下+4, 4以上-4）ができるように番号を振っている。
        '  -+---+-      ●   +++   : ビューア
        '  2|+++|3
        '-- |+++| --
        '  6|+++|7
        '  -+---+-
        '   |4 5|
        'M2   |   M3

        Dim workArea As Rectangle = DirectCast(Owner, FormMain_Viewer).UsingScreen '拡大鏡を使用している作業エリア
        Dim mousePosition As Integer 'マウスカーソルの位置(M0~M3)
        Dim arrangementRank() As Integer '情報パネルを表示する位置のランキング(0~7)
        Dim newRect As New Rectangle With {.Size = Me.Size} '情報パネルの新しい表示位置とサイズ

        '=== ビューアの中心から見てマウスカーソルがある方向 ===
        Dim center As Point = New Point(Owner.Left + Owner.Width \ 2, Owner.Top + Owner.Height \ 2) 'ビューアの中心
        Dim mouse As Point = Windows.Forms.Cursor.Position '現在のマウスカーソル座標

        If mouse.Y > center.Y Then mousePosition = 2
        If mouse.X > center.X Then mousePosition += 1

        '=== 情報パネルの配置の理想順 ===
        'ここで、回避モードごとに順位付けを変更する手もある。
        Select Case mousePosition
            Case 0
                arrangementRank = New Integer() {4, 7, 5, 6, 3, 0, 1, 2}
            Case 1
                arrangementRank = New Integer() {5, 6, 4, 7, 2, 1, 0, 3}
            Case 2
                arrangementRank = New Integer() {0, 3, 1, 2, 7, 4, 5, 6}
            Case Else
                arrangementRank = New Integer() {1, 2, 0, 3, 6, 5, 4, 7}
        End Select

        '=== 移動可能な場所を選ぶ ===
        For Each position As Integer In arrangementRank

            '--- 吸着処理 ---
            'ビューアが作業領域の左右端にくっついたとき、情報パネルも端に寄せる（必須ではない） → 理想順の左右位置を反転することで簡易的に実現する。
            If Owner.Left = workArea.Left OrElse Owner.Right = workArea.Right Then position += If(position Mod 2 = 0, 1, -1)

            '--- 表示位置を算出 ---
            Select Case position
                Case 0
                    newRect.Location = New Point(Owner.Left, Owner.Top - Me.Height)
                Case 1
                    newRect.Location = New Point(Owner.Right - Me.Width, Owner.Top - Me.Height)
                Case 2
                    newRect.Location = New Point(Owner.Left - Me.Width, Owner.Top)
                Case 3
                    newRect.Location = New Point(Owner.Right, Owner.Top)
                Case 4
                    newRect.Location = New Point(Owner.Left, Owner.Bottom)
                Case 5
                    newRect.Location = New Point(Owner.Right - Me.Width, Owner.Bottom)
                Case 6
                    newRect.Location = New Point(Owner.Left - Me.Width, Owner.Bottom - Me.Height)
                Case 7
                    newRect.Location = New Point(Owner.Right, Owner.Bottom - Me.Height)
            End Select

            '--- “作業領域内に収まる”且つ“情報パネルにマウスカーソルが重ならない”か確認 ---
            If workArea.Contains(newRect) AndAlso Not newRect.Contains(mouse) Then Exit For '※※条件を満たせたら確定。満たせないなら次の順位へ。

        Next

        '=== 移動 ===
        Me.Location = newRect.Location

    End Sub





    '|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' 2点間の距離を求める
    ''' </summary>
    ''' <param name="Point1">点1の座標</param>
    ''' <param name="Point2">点2の座標</param>
    ''' <returns>距離</returns>
    ''' <remarks></remarks>
    Protected Shared Function GetDistance_P2P(ByVal Point1 As Point, ByVal Point2 As Point) As Double

        Return Math.Sqrt((Point2.X - Point1.X) ^ 2 + (Point2.Y - Point1.Y) ^ 2)

    End Function

    ''' <summary>
    ''' 角度を求める
    ''' </summary>
    ''' <param name="Center">中心の座標</param>
    ''' <param name="Locate">角度をなす点の座標（相対座標に変換されます）</param>
    ''' <returns>中心点の右を0度として時計回りの角度を返す。</returns>
    ''' <remarks>中心点は、90度となる。</remarks>
    Protected Shared Function GetAngle(ByVal Center As Point, ByRef Locate As Point) As Double

        Dim angle As Double '角度

        Locate = CovertLocate(Center, Locate) '中心点から見た点《Locate》の相対座標

        With Locate
            If .X = 0 Then
                angle = Math.PI / 2
            Else
                angle = Math.Atan(.Y / .X)
                If angle < 0 Then angle = angle + Math.PI
            End If
            If .Y < 0 OrElse (.Y = 0 AndAlso .X < 0) Then angle = angle + Math.PI
        End With

        angle = angle / Math.PI * 180 'ラジアン→1周360

        Return angle

    End Function

    ''' <summary>
    ''' 絶対座標を相対座標に変換する
    ''' </summary>
    ''' <param name="CardinalPoint">基点</param>
    ''' <param name="Locate">座標</param>
    ''' <returns>基点から見た相対座標</returns>
    ''' <remarks></remarks>
    Protected Shared Function CovertLocate(ByVal CardinalPoint As Point, ByVal Locate As Point) As Point

        Locate -= CardinalPoint
        Return Locate

    End Function

End Class